/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.exporter.test;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.backup.ImportExportService;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.MembershipType;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.organization.UserProfile;
import org.exoplatform.test.BasicTestCase;
/**
 * Thu, May 15, 2004    
 * @author: Tuan Nguyen
 * @version: $Id: TestExportService.java,v 1.7 2004/07/24 16:38:59 tuan08 Exp $
 * @since: 0.0
 * @email: tuan08@yahoo.com
 */
public class TestImportExportService extends BasicTestCase {
	
  private static ImportExportService service_ ;
	private static  OrganizationService orgService_ ;
  static String USER  = "demo" ;
  
  private boolean test = true;
	
  public TestImportExportService(String name) {
    super(name);
  }

  public void setUp() throws Exception {   
    if( !test) return;
    PortalContainer manager  = PortalContainer.getInstance();
    service_ = (ImportExportService) manager.getComponentInstanceOfType(ImportExportService.class) ;
    orgService_ = (OrganizationService) manager.getComponentInstanceOfType(OrganizationService.class);    
  }

  public void tearDown() throws Exception { 
    if( !test) return;
  	PortalContainer manager  = PortalContainer.getInstance();
  	HibernateService hservice = 
  		(HibernateService) manager.getComponentInstanceOfType(HibernateService.class) ;
  	hservice.closeSession();
  }  

  public void testExportData() throws Exception { 
    test = false;
    if( !test) return;
    try{
      User user =  createUser( "thuannd");
      createDataUser( user);  
      Group g = createGroup( "group1_test");
      MembershipType mt = createMembershipType( "membershiptype_test");    
      orgService_.getMembershipHandler().linkMembership( user, g, mt, true);
    }catch( Exception exp){
      
    }
    
    service_.exportServiceData() ;    
    System.err.println( service_.getExportLogger().getTextSummary()) ;   

    service_.exportUserData() ;
    System.err.println(service_.getExportLogger().getTextSummary()) ; 
  }
  
  public void testImportData() throws Exception {    
    if( !test) return;
    
    Group group = orgService_.getGroupHandler().findGroupById( "/group1_test");    
//    assertEquals( group.getGroupName(), "group1_test"); 
    
    MembershipType mt = 
      orgService_.getMembershipTypeHandler().findMembershipType( "membershiptype_test");    
    
//    orgService_.getGroupHandler().removeGroup( group, true);
//    orgService_.getMembershipTypeHandler().removeMembershipType( mt.getName(), true);
//    
    service_.importServiceData() ;
    System.err.println(service_.getImportLogger().getTextSummary()) ;
    
    group = orgService_.getGroupHandler().findGroupById( "/group1_test");    
    assertEquals( group.getGroupName(), "group1_test");  
    
    mt = 
      orgService_.getMembershipTypeHandler().findMembershipType( "membershiptype_test");
    assertEquals("Expect mebershiptype is:", "membershiptype_test", mt.getName());    

    service_.importUserData( USER) ;
    User user = orgService_.getUserHandler().findUserByName(USER) ;
    assertTrue("user is not null" , user != null) ;
    System.err.println( service_.getImportLogger().getTextSummary()) ;    
    
    orgService_.getUserHandler().removeUser( "thuannd", true) ;
    User u = orgService_.getUserHandler().findUserByName( "thuannd");   
    assertTrue("Found user instance", u == null);
    
    service_.importUserData( "thuannd") ;
    u = orgService_.getUserHandler().findUserByName( "thuannd");   
    assertTrue("Found user instance", u != null);
    UserProfile up = orgService_.getUserProfileHandler().findUserProfileByName( "thuannd");
    assertEquals("Expect profile is updated: user.gender is ", 
                                       "male", up.getUserInfoMap().get("user.gender"));
    
    Membership membership = orgService_.getMembershipHandler().findMembershipByUserGroupAndType(
                   "thuannd", "/group1_test", "membershiptype_test");
    assertTrue("Memberships : ",membership != null );
  }
  
//  public void testTransformServiceData() throws Exception {
//    if( !test) return;
//    service_.transformServiceData() ;
//    
//    service_.transformUserData() ;
//  }
  
  public Group createGroup(String groupName) throws Exception {    
    Group groupParent = orgService_.getGroupHandler().createGroupInstance() ;
    groupParent.setGroupName( groupName);
    groupParent.setDescription("This is description");
    orgService_.getGroupHandler().createGroup( groupParent, true);   
    return groupParent;
  }
  
  public User createUser(String userName) throws Exception {   
    User user = orgService_.getUserHandler().createUserInstance() ;
    user.setUserName(userName) ;
    user.setPassword("default") ;
    user.setFirstName("default") ;
    user.setLastName("default") ;
    user.setEmail("exo@exoportal.org") ;    
    orgService_.getUserHandler().createUser(user, true);
    return user ;
  }
  
  public MembershipType createMembershipType(String name) throws Exception {
    MembershipType mt = orgService_.getMembershipTypeHandler().createMembershipTypeInstance();
    mt.setName( name) ;
    mt.setDescription("This is a test") ;
    mt.setOwner("exo") ;    
    orgService_.getMembershipTypeHandler().createMembershipType(mt, true);
    return mt;
  }
  
  private User createDataUser( User u) throws Exception {
    UserProfile up = orgService_.getUserProfileHandler().findUserProfileByName( u.getUserName());
    up.getUserInfoMap().put("user.gender", "male");
    orgService_.getUserProfileHandler().saveUserProfile(up, true);    
    return u;
  }
  
 
  //----------------------------------------------------------------------------------------------
  
//public void testTransformUserData() throws Exception {
//service_.transformUserData() ;
//}
//  
  
//  public void testImportServiceData() throws Exception {
//  	service_.importServiceData() ;
//  	System.err.println(service_.getImportLogger().getTextSummary()) ;
//  }
//
//  protected String getDescription() {
//    return "Test Export Service" ;
//  }
}