/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.exporter.test;

import java.io.File;
import java.util.Collection;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.backup.ImportExportService;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.MembershipType;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.organization.UserProfile;
import org.exoplatform.test.BasicTestCase;

/**
 * Created by The eXo Platform SARL
 * Author : Hoa  Pham
 *          hoapham@exoplatform.com,phamvuxuanhoa@yahoo.com
 * Nov 15, 2005
 */
public class TestImportExportServiceModify extends BasicTestCase {
  
  private static ImportExportService service_ ;
  private static  OrganizationService orgService_ ;
  static String defaultName  = "demo" ;
  static String Group1 = "Group1" ;
  static String Group2 = "Group2" ;
  static String uName_1 = "userName_1";
  static String uName_2 = "userName_2" ;
  static String mType_1 = "MembershipType_1" ;
  static String mType_2 = "MembershipType_2" ;   
    
  private Group group1, group2, defaultGroup;  
  private MembershipType mType1,mType2, defaultType ;
  private User user1, user2 ,defaultUser; 
    
  public TestImportExportServiceModify(String name) {
    super(name);
  }

  public void setUp() throws Exception {
    PortalContainer manager  = PortalContainer.getInstance();
    service_ = (ImportExportService) manager.getComponentInstanceOfType(ImportExportService.class) ;
    orgService_ = (OrganizationService) manager.getComponentInstanceOfType(OrganizationService.class);
  }
  
  public void tearDown() throws Exception {    
    PortalContainer manager  = PortalContainer.getInstance();
    HibernateService hservice = 
      (HibernateService) manager.getComponentInstanceOfType(HibernateService.class) ;
    hservice.closeSession();
  }  
  
  public void testExportImportSingleServiceData() throws Exception {    
    prepareServiceData() ;
    /*---------------get all groups and membershiptypes before export/import data*/     
    Group parent = orgService_.getGroupHandler().findGroupById("/") ;
    Collection<Group>  gBefore = orgService_.getGroupHandler().findGroups(parent) ;        
    Collection<MembershipType> memTypeBefore = orgService_.getMembershipTypeHandler().findMembershipTypes() ;

    System.out.println("--------- Begin Export Service data-----------") ; 
    /* --Expect result: found a ZipFile with name:organization.zip
     * --------and it's entries are: Group1.xml/Group1.xml/users.xml(default data)
    * -------------and: mType_1.xml/mType_2.xml/member.xml(defaul data)-------------*/
    String[] groupEntrys = {Group1+".xml",Group2+".xml","users.xml"} ;
    String[] memTypeEntrys = {mType_2+".xml",mType_1+".xml","member.xml"} ;
    service_.exportServiceData("organization") ;
    ZipFile zfile= getZipServiceData("organization") ;
    assertTrue("Expect Group entries is found:"+Group1, hasEntry(zfile,groupEntrys)) ;            
    assertTrue("Expect MemType entries is found:"+mType_2, hasEntry(zfile,memTypeEntrys)) ;
    zfile.close() ;
    System.out.println("------------- Export service data susscessful----------------") ;
        
    System.out.println("------------- Begin Import Service data----------------") ;
    service_.importServiceData("organization") ;
    /*-------Get data after export/import and compare with data that-------- 
     *  --------got berfore export-import to make sure they are same-----------*/
    parent = orgService_.getGroupHandler().findGroupById("/") ;
    Collection<Group>  gAfter = orgService_.getGroupHandler().findGroups(parent) ;    
    assertCollection(gBefore,gAfter,new GroupComparator()) ;
    
    Collection<MembershipType> memTypeAfter = orgService_.getMembershipTypeHandler().findMembershipTypes() ;
    assertCollection(memTypeBefore,memTypeAfter,new MembershipTypeComparator()) ;
    System.out.println("------------- Import Service Data sucsscefull----------------") ;
    
    }
  
  public void testExportImportSingleUserData() throws Exception {
    prepareUserData() ;        
    /*---------------Assgin user1 to group1 with mType1------*/
    orgService_.getMembershipHandler().linkMembership(user1,group1,mType1,true) ;
    /*---------------Assgin user1 to group1 with mType2------*/
    orgService_.getMembershipHandler().linkMembership(user1,group1,mType2,true) ;
    /*---------------Assgin user1 to group2 with mType2------*/
    orgService_.getMembershipHandler().linkMembership(user1,group2,mType2,true) ;
    /*--------- get all data relation with user before import/export------------*/
    Collection<Membership> memBefore = orgService_.getMembershipHandler().findMembershipsByUser(uName_1) ;    
    Collection<Group> gBefore = orgService_.getGroupHandler().findGroupsOfUser(uName_1) ;
    
    System.out.println("--------- Begin Export single user's data-----------") ; 
    /* --Expect result: found a ZipFile with name:user1.userName.zip
     * --------and it's entries are: Memberships.xml/User.xml/UserProfile.xml ---- */    
    service_.exportUserData(user1.getUserName()) ;    
    ZipFile zfile = getZipUserData(uName_1) ;    
    assertTrue("Expect 'memberships.xml' entry is found", hasEntry(zfile,"Memberships.xml")) ;
    assertTrue("Expect 'User.xml' entry is found", hasEntry(zfile,"User.xml")) ;
    assertTrue("Expect 'UserProfile.xml' entry is found", hasEntry(zfile,"UserProfile.xml" )) ;
    zfile.close() ;
    System.out.println("---------Export User data successful-----------") ;   
    
    System.out.println("------- Begin Import User data  --------------") ;    
    service_.importUserData(uName_1) ;
    User u = orgService_.getUserHandler().findUserByName(uName_1) ;
    assertTrue("Expect user has found:",u!=null) ;
    assertTrue("Expect user's info is correct:",u.getFirstName().equalsIgnoreCase("default")) ;
    /*-------------find User Profile and check it----------------*/
    UserProfile profile = orgService_.getUserProfileHandler().findUserProfileByName(uName_1) ;
    assertTrue("Expect user profile has found:", profile.getUserName().equalsIgnoreCase(uName_1)) ;
    assertEquals("Expect profile's info is correct: user.gender is ","male", profile.getUserInfoMap().get("user.gender"));
    /*------------get all data relate with user after export/import and compare---------
     * ----------with data that get before import/export to make sure they are same-----*/
    Collection<Group>  gAfter= orgService_.getGroupHandler().findGroupsOfUser(uName_1) ;    
    assertCollection(gBefore,gAfter,new GroupComparator()) ;
        
    Collection<Membership> memAfter = orgService_.getMembershipHandler().findMembershipsByUser(uName_1) ;
    assertCollection(memBefore,memAfter, new MembershipComparator()) ;
    System.out.println("----------Import single user data succesful -----------------") ;        
   }
  
  public void testExportImportAllUserData() throws Exception {
    prepareUserData() ;
    /*---------get all group and membership relation with user1 before export-import--*/
    Collection<Group> groupBeforeOfU_1 = orgService_.getGroupHandler().findGroupsOfUser(uName_1) ;        
    Collection<Membership> memBeforeOfU_1 = orgService_.getMembershipHandler().findMembershipsByUser(uName_1) ;
    
    /*---------------Assgin user2 to group1 with mType1------*/
    orgService_.getMembershipHandler().linkMembership(user2,group1,mType1,true) ;
    /*---------------Assgin user2 to group1 with mType2------*/
    orgService_.getMembershipHandler().linkMembership(user2,group2,mType1,true) ;
    /*---------Get all group and membership relation with user2 before export-import--*/
    Collection<Group> groupBeforeOfU_2 = orgService_.getGroupHandler().findGroupsOfUser(uName_2) ;
    Collection<Membership> memBeforeOfU_2 = orgService_.getMembershipHandler().findMembershipsByUser(uName_2) ;
    
        /*---------------Assgin demo{default user} to group1 with mType1------*/
    orgService_.getMembershipHandler().linkMembership(defaultUser,group1,mType1,true) ;
    /*---------Get all group and membership relation with "demo" before export-import--*/
    Collection<Group> groupBeforeOfDefault = orgService_.getGroupHandler().findGroupsOfUser("demo") ;
    Collection<Membership> memBeforeOfDefault = orgService_.getMembershipHandler().findMembershipsByUser("demo") ;
    
    System.out.println("--------- Begin Export all user's data-----------") ;
    /* --Expect result: found 3 ZipFile with name:user1.zip, user2.zip,demo.zip(default user)
     * --------and in a file has 3 entries: Memberships.xml/User.xml/UserProfile.xml--- */ 
    String[] entryNames = {"Memberships.xml","User.xml","UserProfile.xml"} ;
    service_.exportUserData() ;
    ZipFile zfile = getZipUserData(uName_1) ;
    assertTrue("Expect entrys are found in zipfile of user:"+uName_1,hasEntry(zfile,entryNames)) ;
    zfile = getZipUserData(uName_2) ;
    assertTrue("Expect entrys are found in zipfile of user :"+uName_2,hasEntry(zfile,entryNames)) ;
    zfile = getZipUserData("demo") ;
    assertTrue("Expect entrys are found in zipfile of user :demo",hasEntry(zfile,entryNames)) ; 
    zfile.close() ;
    System.out.println("--------- Success export all user's data-----------") ;
        
    System.out.println("--------- begin import all user's data-----------") ;
    service_.importUserData() ;
    /*--------------Expect all data of USER_1 before/after export/import are same-----*/
    Collection<Group> groupAfterOfU_1 = orgService_.getGroupHandler().findGroupsOfUser(uName_1) ;      
    assertCollection(groupBeforeOfU_1,groupAfterOfU_1,new GroupComparator()) ;
    
    Collection<Membership> memAfterOfU_1 = orgService_.getMembershipHandler().findMembershipsByUser(uName_1) ;
    assertCollection(memBeforeOfU_1,memAfterOfU_1,new MembershipComparator()) ;
    
    /*---Expect all data of USER_2 before/after export/import are same-----*/
    Collection<Group> groupAfterOfU_2 = orgService_.getGroupHandler().findGroupsOfUser(uName_2) ;
    assertCollection(groupBeforeOfU_2,groupAfterOfU_2,new GroupComparator()) ;
    
    Collection<Membership> memAfterOfU_2 = orgService_.getMembershipHandler().findMembershipsByUser(uName_2) ;
    assertCollection(memBeforeOfU_2,memAfterOfU_2, new MembershipComparator()) ;
    
    /*---Expect all data of DEFAULT USER(demo) before/after export/import are same-----*/
    Collection<Group> groupAfterOfDefault = orgService_.getGroupHandler().findGroupsOfUser("demo") ;
    assertCollection(groupBeforeOfDefault,groupAfterOfDefault,new GroupComparator()) ;
    
    Collection<Membership> memAfterOfDefault = orgService_.getMembershipHandler().findMembershipsByUser("demo") ;
    assertCollection(memBeforeOfDefault,memAfterOfDefault,new MembershipComparator()) ;    
    System.out.println("---------import all user's data successful -----------") ;    
  }
  public void testExportImportAllServiceData() throws Exception {    
    prepareServiceData() ;
    /*---------------get all data before export/import ---------
     * ------------------------service: organization service----------------*/
    Group parent = orgService_.getGroupHandler().findGroupById("/") ;
    Collection<Group>  gBefore = orgService_.getGroupHandler().findGroups(parent) ;        
    Collection<MembershipType> memTypeBefore = orgService_.getMembershipTypeHandler().findMembershipTypes() ;
    /*--------------------------Export all service data--------------     
     /* ---------Expect result: found a ZipFile with name:organization.zip--------
     * --------and it's entries are: Group1.xml/Group2.xml/users.xml(default data)
    * -------------and: mType_1.xml/mType_2.xml/member.xml(defaul data)-------------*/
    
    String[] entryNames = {"Group1.xml","Group2.xml","users.xml","mType_1.xml","mType_2.xml","member.xml"} ;
    System.out.println("----------begin export data of all services -----------------") ;
    service_.exportServiceData() ;
    ZipFile zfile= getZipServiceData("organization") ;
    assertTrue("Organization.zip service is found with it's entries",hasEntry(zfile,entryNames)) ;
    zfile.close() ;
    System.out.println("----------Export data of all services successfull-----------------") ;
        
    System.out.println("----------begin import data of all services-----------------") ;    
    service_.importServiceData() ;
    /*---Expect all groups before/after export/import are same-----*/
    parent = orgService_.getGroupHandler().findGroupById("/") ;
    Collection<Group>  gAfter = orgService_.getGroupHandler().findGroups(parent) ;    
    assertCollection(gBefore,gAfter,new GroupComparator()) ;
    /*---Expect all MembershipType before/after export/import are same-----*/
    Collection<MembershipType> memTypeAfter = orgService_.getMembershipTypeHandler().findMembershipTypes() ;
    assertCollection(memTypeBefore,memTypeAfter,new MembershipTypeComparator()) ;
    System.out.println("----------import data of all services are successfull-----------------") ;
  }
  
  public void prepareServiceData() throws Exception {
    defaultGroup = orgService_.getGroupHandler().findGroupById("/users") ;    
    if(orgService_.getGroupHandler().findGroupById("/"+Group1)==null) { group1 = createGroup( Group1); }
    else {group1 = orgService_.getGroupHandler().findGroupById("/"+Group1) ; }
    
    if(orgService_.getGroupHandler().findGroupById("/"+Group2)==null) { group2 = createGroup( Group2) ; }
    else {group2= orgService_.getGroupHandler().findGroupById("/"+Group2); }
    
    defaultType = orgService_.getMembershipTypeHandler().findMembershipType("member") ;
    if(orgService_.getMembershipTypeHandler().findMembershipType(mType_1)==null) { 
      mType1 = createMembershipType(mType_1); 
    }else {
      mType1 = orgService_.getMembershipTypeHandler().findMembershipType(mType_1) ;
    }
    
    if(orgService_.getMembershipTypeHandler().findMembershipType(mType_2)==null) {
      mType2 = createMembershipType(mType_2); 
    }else {
      mType2 = orgService_.getMembershipTypeHandler().findMembershipType(mType_2) ;
    }          
  }
  
  public void prepareUserData() throws Exception{
    prepareServiceData() ;
    defaultUser = orgService_.getUserHandler().findUserByName("demo") ;
    if(orgService_.getUserHandler().findUserByName(uName_1)==null) {
      user1 =  createUser(uName_1);
      createDataUser(user1);
    }else {
      user1 = orgService_.getUserHandler().findUserByName(uName_1) ;
      Collection<Membership> mem = orgService_.getMembershipHandler().findMembershipsByUser(uName_1) ;          
    }
    
    if(orgService_.getUserHandler().findUserByName(uName_2)==null) {
      user2= createUser(uName_2) ;    
      createDataUser(user2) ; 
    }else {
      user2 = orgService_.getUserHandler().findUserByName(uName_2) ;
    }        
  }
            
  public Group createGroup(String groupName) throws Exception {    
    Group groupParent = orgService_.getGroupHandler().createGroupInstance() ;
    groupParent.setGroupName( groupName);
    groupParent.setDescription("This is description");
    orgService_.getGroupHandler().createGroup( groupParent, true);   
    return groupParent;
  }
      
  public MembershipType createMembershipType(String name) throws Exception {
    MembershipType mt = orgService_.getMembershipTypeHandler().createMembershipTypeInstance();
    mt.setName( name) ;
    mt.setDescription("This is a test") ;
    mt.setOwner("exo") ;    
    orgService_.getMembershipTypeHandler().createMembershipType(mt, true);
    return mt;
  }
  
  public User createUser(String userName) throws Exception {   
    User user = orgService_.getUserHandler().createUserInstance() ;
    user.setUserName(userName) ;
    user.setPassword("default") ;
    user.setFirstName("default") ;
    user.setLastName("default") ;
    user.setEmail("exo@exoportal.org") ;
    orgService_.getUserHandler().createUser(user, true);
    return user ;
  }
  
  private User createDataUser( User u) throws Exception {
    UserProfile up = orgService_.getUserProfileHandler().findUserProfileByName( u.getUserName());
    up.getUserInfoMap().put("user.gender", "male");
    orgService_.getUserProfileHandler().saveUserProfile(up, true);    
    return u;
  }
  
  private ZipFile getZipServiceData(String serviceName) throws Exception{
    String zipFileName = service_.getServiceDataDirectory() + serviceName+ ".zip" ;
    ZipFile zipFile  = new ZipFile(new File(zipFileName)) ;
    return zipFile ;    
  }
   
  private ZipFile getZipUserData(String userName) throws Exception{
    String zipFileName = service_.getUserDataDirectory() + userName+ ".zip" ;
    ZipFile zipFile  = new ZipFile(new File(zipFileName)) ;
    return zipFile ;    
  }
  
  private boolean hasEntry(ZipFile zfile, String entryName) throws Exception {
    Enumeration e =  zfile.entries() ;
    while(e.hasMoreElements()) {
      ZipEntry entry = (ZipEntry) e.nextElement() ;
      if((entry.getName().endsWith(entryName))&&(entry.getSize()>0)) {return true ;}
     }    
    return false ;
  }
  
  private boolean hasEntry(ZipFile zfile, String[] entryNames) throws Exception {
    Enumeration e =  zfile.entries() ;
    while(e.hasMoreElements()) {
      ZipEntry entry = (ZipEntry) e.nextElement() ;
      for(int i=0;i<entryNames.length;i++) {
        if((entry.getName().endsWith(entryNames[i]))&&(entry.getSize()>0)) {return true ; }
      }      
     }    
    return false ;
  }
    
  public static class GroupComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      Group g1 = (Group) o1 ;
      Group g2 = (Group) o2 ;
      if(g1.getId().equals(g2.getId()))  return 0 ;
      return -1;
    }
  }
  
  public static class MembershipTypeComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      MembershipType mType1 = (MembershipType) o1 ;
      MembershipType mType2 = (MembershipType) o2;
      if(mType1.getName().equals(mType2.getName())) return 0 ;  
      return -1;
    }    
  }  
  public static class MembershipComparator implements Comparator {

    public int compare(Object o1, Object o2) {
      Membership m1 = (Membership) o1 ;
      Membership m2 = (Membership) o2 ;      
      if ((m1.getGroupId().equals(m2.getGroupId()))
        &&(m1.getMembershipType().equals(m2.getMembershipType()))
        &&(m1.getUserName().equals(m2.getUserName()))
        ) {return 0 ;}
      return -1;
    }    
  }
      
//public void testTransformServiceData() throws Exception {
//service_.transformServiceData() ;
//
//service_.transformUserData() ;
//}
}
