/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.cache.impl;

import java.io.Serializable;
import java.util.* ;
import java.lang.ref.SoftReference;

import org.exoplatform.services.cache.CacheListener;
import org.exoplatform.services.cache.ExoCache; 
import org.exoplatform.services.remote.group.CommunicationService;
import org.exoplatform.services.remote.group.Message ;
/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Sat, Sep 13, 2003 @
 * Time: 1:12:22 PM
 */
public class DistributedExoCache  extends LinkedHashMap implements ExoCache {
  private static int DEFAULT_MAX_SIZE =  50 ;
  
  private String name_ ;
  private String label_ ;
  
  private int maxSize_ ;
  private long liveTime_ = -1;
  private int cacheHit_ ;
  private int cacheMiss_ ;
  private boolean replicated_ = false ;
  private CommunicationService communicationService_ ;
  
  public DistributedExoCache(CommunicationService communicationService) {  
    maxSize_  = DEFAULT_MAX_SIZE ; 
    communicationService_ = communicationService ;
  }
  
  public String getName() { return name_ ; }
  public void   setName(String s) {  
    name_ = s ;
  } 
  
  public String getLabel() { 
    if(label_ ==  null){
      if(name_.length() > 30 ){
        String shortLabel = name_.substring(name_.lastIndexOf(".") + 1) ; 
        setLabel(shortLabel) ;
        return  shortLabel;
      }
      return name_ ; 
    }else{
      return label_ ;
    }
  }
  
  public void setLabel(String name) {
    label_ = name ; 
  }
  
  public int  getCacheSize()  { return size() ; }
  
  public int  getMaxSize() { return maxSize_ ; }
  public void setMaxSize(int max) { maxSize_ = max ; }
  
  public long  getLiveTime() { return liveTime_ ; }
  public void  setLiveTime(long period)  { liveTime_ = period ;} 
  
  synchronized public Object get(Serializable name)  {
    CacheSoftReference ref = (CacheSoftReference) super.get(name) ;
    if(ref !=  null)  { 
      if(isExpire(ref)) {
        super.remove(name) ;
        cacheMiss_++ ;
        return null ;
      }
      cacheHit_++ ;
      return ref.get() ;
    }
    cacheMiss_++ ;
    return null ; 
  }
  
  synchronized public Object remove(Serializable name)  throws Exception {
    CacheSoftReference ref = (CacheSoftReference) super.get(name) ;
    broadcast(name, null) ;
    if(ref != null)  {
      if(isExpire(ref)) {
        return null ;
      }
      return ref.get() ;
    }
    return null ;
  }
  
  synchronized public void put(Serializable name, Object obj)  throws Exception {
    long expireTime = -1 ;
    if(liveTime_ > 0) expireTime = System.currentTimeMillis() + liveTime_ ;
    CacheSoftReference ref = new CacheSoftReference(expireTime, obj) ;
    super.put(name, ref) ;
    broadcast(name, obj) ;
  }
  
  synchronized public void clear() {
    super.clear() ;
    try {
      broadcast(null, null) ;
    } catch (Exception ex) {
      ex.printStackTrace() ;
    }
  }
  
  public int getCacheHit()  { return cacheHit_  ;}
  
  public int getCacheMiss() { return cacheMiss_ ; }
  
  public boolean isDistributed()  { return true ; }
  
  public boolean isReplicated() { return replicated_ ; }
  public void    setReplicated(boolean b) { replicated_ = b ; }
  
  synchronized public void synchronize(Serializable key, Object value) throws Exception {
    System.out.println("Synchonize key : " + key + " , value: " + value) ;
    if(key == null) { //invalidate all cache if key is null ;
      super.clear() ;
    } else if(value == null) { //remove the key if value is null ;
      super.remove(key) ;
    } else { //replicate the data
      long expireTime = -1 ;
      if(liveTime_ > 0) expireTime = System.currentTimeMillis() + liveTime_ ;
      CacheSoftReference ref = new CacheSoftReference(expireTime, value) ;
      super.put(key, ref) ;
    }
  }
  
  public void addCacheListener(CacheListener listener) {
   
  }
  
  protected boolean removeEldestEntry(Map.Entry eldest) {
    return size() > maxSize_ ;
  }
  
  private boolean isExpire(CacheSoftReference ref) {
    //-1 mean cache live for ever
    if(liveTime_ < 0) return false ;
    if(System.currentTimeMillis() > ref.getExpireTime()) return true ;
    return false ;
  }
  
  private void broadcast(Serializable key, Object value) throws Exception {
    Message message = 
      communicationService_.createMessage(SynchronizeCacheMessageHandler.IDENTIFIER) ;
    SynchronizeCacheMessage synMessage = null ;
    if(replicated_) {
      synMessage = new SynchronizeCacheMessage(name_, key, (Serializable)value) ;
    } else {
      synMessage = new SynchronizeCacheMessage(name_, key, null) ;
    }
    message.setMessage(synMessage) ;
    communicationService_.broadcast(message, false) ;
  }
  
  static public class CacheSoftReference extends SoftReference {
    private long expireTime_ ;
    
    public CacheSoftReference(long expireTime, Object o) {
      super(o) ;
      expireTime_ = expireTime ;
    }
    
    public long getExpireTime() { return expireTime_ ;  }
  }

  public void clearCache() throws Exception {
    // TODO Auto-generated method stub
    
  }

  public List getCachedObjects() {
    // TODO Auto-generated method stub
    return null;
  }
}