/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.services.scheduler.test;

import java.util.Date;
import java.util.List;
import java.util.ListIterator;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.RootContainer;
import org.exoplatform.container.component.BaseComponentPlugin;
import org.exoplatform.services.scheduler.BaseJob;
import org.exoplatform.services.scheduler.JobContext;
import org.exoplatform.services.scheduler.JobInfo;
import org.exoplatform.services.scheduler.JobSchedulerService;
import org.exoplatform.services.scheduler.PeriodInfo;
import org.exoplatform.test.BasicTestCase;
import org.quartz.JobExecutionContext;
import org.quartz.JobListener;
import org.quartz.Trigger;
import org.quartz.TriggerListener;


/**
 * Created by The eXo Platform SARL
 * Author : Hoa  Pham
 *          hoapham@exoplatform.com
 * Oct 5, 2005
 */
public class TestSchedulerService extends BasicTestCase {
  private JobSchedulerService service_ ;
  //private JobListener jobListener_ ;
  
  public void setUp() throws Exception {
     PortalContainer manager  = PortalContainer.getInstance();
     service_ = (JobSchedulerService) manager.getComponentInstanceOfType(JobSchedulerService.class) ;
  }

  public void tearDown() throws Exception { 
    RootContainer.setInstance(null) ;
  }

  public void testSchedulerSevice() throws Exception {
    assertTrue("JobScheduler is not deployed correctly", service_ != null) ;
    Thread.sleep(1000) ;
    Date startTime = new Date(System.currentTimeMillis() + 1000) ;
    Date endTime = null ;
    JobInfo jinfo ;
    
    //Test addJob(JobInfo, Date startTime) and TestTriggerListener()
    resetTestEnvironment() ;  
    jinfo = new JobInfo("TestAddJob",null/*default group*/,AJob.class) ;
    AJob.expectExecuteTime_ = startTime ;
    service_.addGlobalTriggerListener(new TestTriggerListener()) ;
    service_.addJob(jinfo, startTime);
    Thread.sleep(1100) ;
    assertEquals("task has been run once" , 1, AJob.counter_) ;
    assertEquals("TestTriggerListener detect one task", 1, TestTriggerListener.countTriggerFired_) ;
    assertTrue("The task is triggered  at the correct time  within 500ms margin", AJob.errorCounter_ == 0) ;
    
    /*Test addJobPeriod(JobInfo jinfo,JobPeriodInfor pinfor)
     * Job will fire after 3 second, repeat forever, 2 second pause between repeat fire */
    resetTestEnvironment() ;
    jinfo = new JobInfo("TestJobPeriod",null/*default group*/,AJob.class) ;
    startTime = new Date(System.currentTimeMillis() + 1000) ;
    PeriodInfo jpinfo = new PeriodInfo(startTime,null /*endTime*/,2/*repeatCount*/,1000/*period*/) ;
    service_.addPeriodJob(jinfo, jpinfo) ;
    Thread.sleep(2100) ;
    assertEquals("task has been run exactly two times" ,2 , AJob.repeatCounter_) ;

    /*Test addJobPeriod(JobInfo jinfo,JobPeriodInfor pinfor)
     * Job will fire after 1 second, stop after 2 second, 1 second pause between repeat fire */
    resetTestEnvironment() ;
    long currentTime = System.currentTimeMillis()  ;
    jinfo = new JobInfo("TestJobPeriod",null/*default group*/,AJob.class) ;
    startTime = new Date( currentTime + 1000) ;
    endTime = new Date(currentTime + 2100) ;
    jpinfo=new PeriodInfo(startTime, endTime, 0/*repeatCount*/, 1000/*period*/) ;
    service_.addPeriodJob(jinfo, jpinfo) ;
    Thread.sleep(2500) ;
    assertEquals("task has been run exactly two times" ,2 , AJob.repeatCounter_) ;
           
    /* test test addPeriodJob(JobInfo jinfo,JobperiodInfo pinfo)
     * Job will run imediately, and exactly two times, 1 second pause between repeat run */
    resetTestEnvironment() ;
    jinfo = new JobInfo("ImediatelyPeriodJob",null/*default group*/,AJob.class) ;
    jpinfo=new PeriodInfo(null/*startTime*/, null/*endTime*/, 2/*repeatCount*/, 500/*period*/) ;
    service_.addPeriodJob(jinfo, jpinfo) ;
    Thread.sleep(1100) ;
    assertEquals("task has been run exactly two times" ,2 , AJob.repeatCounter_) ;
    service_.removeJob(jinfo);
    /* test addCronJob(JobInfor jinfo, String exp);
     * Job will fire at 10: am every day "0 15 10 ? * *"*/
    resetTestEnvironment() ;
    String exp = "0 25 11 ? * *" ;
    jinfo = new JobInfo("TestCronJob",null/*default group*/,AJob.class) ;
    service_.addCronJob(jinfo,exp) ;
    assertEquals("task has been run exactly one times at "+ exp, 0, AJob.repeatCounter_) ;
    service_.removeJob(jinfo);
    
    /* test test addPeriodJob(JobInfo jinfo,JobperiodInfo pinfo)
     * Job will run imediately, and forever, 1 second pause between repeat run */
    resetTestEnvironment() ;
    jinfo = new JobInfo("ForeverPeriodJob",null/*default group*/,AJob.class) ;
    jpinfo=new PeriodInfo(null/*startTime*/, null/*endTime*/, 0/*repeatCount*/, 500/*period*/) ;
    service_.addPeriodJob(jinfo, jpinfo) ;
    Thread.sleep(1100) ;
    assertEquals("task has been run forever: " ,3 , AJob.repeatCounter_) ;
    service_.removeJob(jinfo) ;    
  }
  
  public void testJobExecuting() throws Exception {
    JobInfo jinfo = new JobInfo("runningjob","running",RunningJob.class) ;
    service_.addJob(jinfo, new Date()) ;
    Thread.sleep(RunningJob.SLEEP_TIME - 100) ;
    List runningJobs = service_.getAllExcutingJobs() ;
    assertFalse("job excution context is empty",runningJobs.isEmpty());
    // Some infomation about job is running 
    assertEquals("expect one running  job: ", 1, runningJobs.size()) ;
    Thread.sleep(500) ;
  }
  
  public void testgetAvailableJobs() throws Exception {
   List availableJobs = service_.getAllJobs();
    assertEquals("Expect no job inthe queue",0, availableJobs.size())  ;
    //some information about job execution
    Date  firedTime =  new Date(System.currentTimeMillis() + 1000000) ;
    service_.addJob(new JobInfo("queuejob", null/*default group*/,AJob.class), firedTime) ;
    availableJobs = service_.getAllJobs();
    assertEquals("Expect one job inthe queue", 1, availableJobs.size())  ;
  }
  
  public void testListener() throws Exception  {
    List jobListener ;
    boolean b;
    
    //test getAllGlobalJobListener()
    jobListener = service_.getAllGlobalJobListener(); 
    JobListener jlistener;
    ListIterator iter = jobListener.listIterator();
    System.out.println("===========> all job listener: ");
    while(iter.hasNext()) {
      jlistener = (JobListener)iter.next() ;
      System.out.println("==========>Job listener name:" + jlistener.getName()) ;
    }
    
    //test removeGlobalJobListener()
    System.out.println("=============> remove a global remove listener: ");
    jlistener = service_.getGlobalJobListener("JobContextConfigListener") ;
    b = service_.removeGlobalJobListener("JobContextConfigListener") ;
    assertTrue("remove job listener is:",b);
    
    //test getAllTriggerListener()
    List triggerListener ;
    triggerListener = service_.getAllGlobalTriggerListener() ;
    TriggerListener tlistener ;
    System.out.println("=========> all trigger listener: ");
    ListIterator iterator = triggerListener.listIterator() ;
    while(iterator.hasNext()){
      tlistener = (TriggerListener) iterator.next() ;
      System.out.println("=======> TriggerListener name: " + tlistener.getName()) ;
    }
    
    //test removeGlobalTriggerListener()
    System.out.println("======> remove a global trigger listener");
    tlistener = service_.getGlobalTriggerListener("TestTriggerListener") ;
    b = service_.removeGlobaTriggerListener("TestTriggerListener") ;
    assertTrue("remove job listener is: ", b);
  } 
  
  static  private  void resetTestEnvironment() {
    AJob.reset() ;
    TestTriggerListener.countTriggerComplete_ = 0 ; 
  }
  
  static public class  TestTriggerListener 
    extends BaseComponentPlugin implements  TriggerListener {
    
    static int countTriggerFired_ = 0 ;
    static int countTriggerComplete_ = 0 ;
    
    public String getName() { return "TestTriggerListener";  }

    public void triggerFired(Trigger arg0, JobExecutionContext arg1) {
      countTriggerFired_++ ;
      System.out.println("trigger fired.......................") ;
    }

    public boolean vetoJobExecution(Trigger arg0, JobExecutionContext arg1) {
      return false;
    }

    public void triggerMisfired(Trigger arg0) {
      System.out.println("calll  TestTask.......................") ;
    }

    public void triggerComplete(Trigger arg0, JobExecutionContext arg1, int arg2) {
      countTriggerComplete_++ ;
      System.out.println("trigger complete.......................") ;      
    }  
  }
  
  static public class RunningJob extends BaseJob { 
    static long SLEEP_TIME = 1000 ;
    public void  execute(JobContext  context) throws Exception {
        Thread.sleep(SLEEP_TIME) ;
    }
  }

}
  


  