/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.document.impl;
import java.io.InputStream;
import java.util.Calendar;

import org.apache.poi.hssf.usermodel.*;

/**
 * Created by The eXo Platform SARL
 * Author : Phung Hai Nam
 *          phunghainam@gmail.com
 * Oct 21, 2005
 */
public class MSExcelDocumentReader extends BaseDocumentReader {
  
  public String getMimeTypes() { 
    return "application/excel" ;
  }
  
  public String getContentAsText(InputStream is) throws Exception {
    String text = null;
    try {
      HSSFWorkbook wb = new HSSFWorkbook(is);
      for (int i=0; i < wb.getNumberOfSheets(); i++) {
        HSSFSheet sheet = wb.getSheetAt(i);
        if (sheet != null) {
          for (int j=0; j < sheet.getPhysicalNumberOfRows(); j++) {
            HSSFRow row = sheet.getRow(j);
            for(int k=0; k < row.getPhysicalNumberOfCells(); k++) {
              HSSFCell cell = row.getCell((short) k) ;
              switch(cell.getCellType()) {
                case HSSFCell.CELL_TYPE_NUMERIC : {
                  double d = cell.getNumericCellValue();
                  if (isCellDateFormatted(cell)) {
                    Calendar cal = null ;
                    cal.setTime(HSSFDateUtil.getJavaDate(d));
                    String cellText = "" ;
                    cellText= (String.valueOf(cal.get(Calendar.YEAR))).substring(2);
                    cellText = cal.get(Calendar.MONTH)+1 + "/" + cal.get(Calendar.DAY_OF_MONTH) + 
                    "/" +  cellText;
                  } else text =text + d + " \n"; 
                  break;
                }
                case HSSFCell.CELL_TYPE_FORMULA  :
                  text =text + cell.getCellFormula().toString() + " "; 
                  break;  
                case  HSSFCell.CELL_TYPE_BOOLEAN  : 
                  text =text + cell.getBooleanCellValue() + " "; 
                  break;
                case HSSFCell.CELL_TYPE_ERROR  : 
                  text =text + cell.getErrorCellValue() + " ";
                  break;
                case HSSFCell.CELL_TYPE_STRING : 
                  text =text + cell.getStringCellValue().toString() + " ";
                  break ;
                default :
                  break ;
              }    
            }
          }
        }
      }
    }catch(Exception e)  {}
    return text ;
  }
  
  public static boolean isCellDateFormatted(HSSFCell cell) {
    boolean bDate = false;
    double d = cell.getNumericCellValue();
    if( HSSFDateUtil.isValidExcelDate(d) ) {
      HSSFCellStyle style = cell.getCellStyle();
      int i = style.getDataFormat();
      switch(i) {
        case 0xe:
        case 0xf:
        case 0x10:
        case 0x11:
        case 0x12:
        case 0x13:
        case 0x14:
        case 0x15:
        case 0x16:
        case 0x2d:
        case 0x2e:
        case 0x2f:
           bDate = true;
           break;
        default:
          bDate = false;
          break;
       }
     }
     return bDate;
     }
}
