/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.organization.ldap;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import javax.naming.CompositeName;
import javax.naming.Name;
import javax.naming.NameParser;
import javax.naming.NamingEnumeration;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.LdapContext;

import org.exoplatform.services.ldap.LDAPService;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.impl.GroupImpl;

/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Oct 14, 2005
 */
public class BaseHandler {
  
  protected LDAPAttributeMapping ldapAttrMapping_ ;
  protected LDAPService ldapService_ ;
  
  public  BaseHandler(LDAPAttributeMapping ldapAttrMapping, LDAPService ldapService) {
    ldapAttrMapping_ = ldapAttrMapping ;
    ldapService_ =  ldapService ;
  }
  
  protected String getGroupDNFromGroupId(String groupId) {   
    StringBuffer buffer = new StringBuffer();
    String groupParts[] = groupId.split("/");
    for (int x = (groupParts.length - 1); x > 0; x--) {
      buffer.append("ou=" + groupParts[x] + ", ");
    }   
    buffer.append( ldapAttrMapping_.groupsURL);

    return buffer.toString();
  }
  
  protected List<Object> getAttributes(Attributes attributes, String attribute) {
    List<Object> results = new ArrayList<Object>();
    try {
      if (attributes == null)  return results;
      Attribute attr = attributes.get(attribute);
      for (int x = 0; x < attr.size(); x++)  results.add(attr.get(x));  
    } catch (Exception e) {
    }
    return results;
  }
  
  protected String escapeDN(String dn) {
    StringBuffer buf = new StringBuffer(dn.length());
    for (int i = 0; i < dn.length(); i++) {
      char c = dn.charAt(i);
      switch (c) {
      case '\\':
        buf.append("\\5c");
        break;
      case '*':
        buf.append("\\2a");
        break;
      case '(':
        buf.append("\\28");
        break;
      case ')':
        buf.append("\\29");
        break;
      case '\0':
        buf.append("\\00");
        break;
      default:
        buf.append(c);
        break;
      }
    }
    return buf.toString();
  }
  
  protected Group getGroupFromMembershipDN(String membershipDN) throws Exception {
    LdapContext ctx = ldapService_.getLdapContext();
    NameParser parser = ctx.getNameParser("");
    String membershipParts[] = this.explodeDN( parser.parse(membershipDN), false);
    
    StringBuffer buffer = new StringBuffer();
    for (int x = 1; x < membershipParts.length; x++) {
      if (x == membershipParts.length - 1) {
        buffer.append(membershipParts[x]);
      } else {
        buffer.append(membershipParts[x] + ",");
      }
    }
    Group group = this.getGroupByDN(buffer.toString());
    
    return group;
  }
  
  protected Group getGroupByDN(String groupDN) throws Exception {
    LdapContext ctx = ldapService_.getLdapContext();    
    NameParser parser = ctx.getNameParser("");   
    StringBuffer idBuffer = new StringBuffer();
    String parentId = null;
    String baseParts[] = this.explodeDN(parser.parse( ldapAttrMapping_.groupsURL), true);    
    String membershipParts[] = explodeDN(parser.parse(groupDN), true);    
    for (int x = (membershipParts.length - baseParts.length - 1); x > -1; x--) {      
      idBuffer.append("/" + membershipParts[x]);
      if (x == 1) parentId = idBuffer.toString();      
    }    
    if (idBuffer == null) return null;
    Attributes attrs = ctx.getAttributes(groupDN);
    GroupImpl group = new GroupImpl();
    group.setGroupName( membershipParts[0]);
    group.setId( idBuffer.toString());
    group.setDescription( ldapAttrMapping_.getAttributeValueAsString(attrs, "description"));
    group.setLabel( ldapAttrMapping_.getAttributeValueAsString( attrs, "l"));  
    group.setParentId( parentId);
    return group;    
  }  

  protected String[] explodeDN(Name dn, boolean removeTypes){
    String explodedDN[] = null;
    
    Enumeration<String> enumeration = dn.getAll();
    List<String> list = new ArrayList<String>();
    while(enumeration.hasMoreElements()){
      String ldap = enumeration.nextElement();
      if (removeTypes){
        int position = ldap.indexOf("=");
        String value = ldap.substring(position + 1);
        list.add(0, value);
      } else {
        list.add(0, ldap);
      }
    }
    explodedDN = new String[list.size()];
    list.toArray(explodedDN);
    
    //System.out.println( "explodedDN co la  "+explodedDN);
    return explodedDN;
  } 
  
  protected String getDNFromUsername( String username) throws Exception {//ham nay
    String dn = null;
    LdapContext ctx = ldapService_.getLdapContext();   
    String userIdAttribute = ldapAttrMapping_.userUsernameAttr;
    
    SearchControls constraints = new SearchControls();
    constraints.setSearchScope(SearchControls.SUBTREE_SCOPE);
    String attrList[] = { "" };
    constraints.setReturningAttributes(attrList);
    constraints.setDerefLinkFlag(true);
    String filter = null;
    filter = "(" + userIdAttribute + "=" + username + ")";   
    NamingEnumeration<SearchResult> answer = ctx.search( ldapAttrMapping_.userURL, filter, constraints);    
    while (answer.hasMore()) {
      SearchResult sr = answer.next();
      NameParser parser = ctx.getNameParser("");
      Name entryName = parser.parse(new CompositeName( sr.getName()).get(0));
      dn = entryName + "," + ldapAttrMapping_.userURL;
    }
    return dn;
  }
  
  protected void romoveAllSubtree( LdapContext context, String dn){
    try {        
       SearchControls constraints = new SearchControls();
       constraints.setSearchScope( SearchControls.ONELEVEL_SCOPE);      
       NamingEnumeration<SearchResult> results = 
             context.search(  dn, "(objectclass=*)",  constraints);
       while( results.hasMore()){
         SearchResult sr =  results.next();        
         romoveAllSubtree( context, sr.getNameInNamespace());
       }      
       context.destroySubcontext( dn);
      }catch( Exception exp){        
      }
  }
  
}
