/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.organization.ldap;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.naming.CompositeName;
import javax.naming.Name;
import javax.naming.NameNotFoundException;
import javax.naming.NameParser;
import javax.naming.NamingEnumeration;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.LdapContext;

import org.exoplatform.services.ldap.LDAPService;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.MembershipEventListener;
import org.exoplatform.services.organization.MembershipHandler;
import org.exoplatform.services.organization.MembershipType;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.organization.impl.MembershipImpl;
/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Oct 14, 2005
 */
public class MembershipHandlerImpl extends BaseHandler implements MembershipHandler {
  
  protected List<MembershipEventListener> listeners_;

  public MembershipHandlerImpl(LDAPAttributeMapping ldapAttrMapping, LDAPService ldapService) {
    super(ldapAttrMapping, ldapService) ;
    listeners_ = new ArrayList<MembershipEventListener>(3);
  }
  
  public void addMembershipEventListener(MembershipEventListener listener) {
    listeners_.add(listener);
  }

  final public Membership createMembershipInstance() {  return new MembershipImpl(); }

  public void createMembership(Membership m, boolean broadcast) throws Exception {    
    LdapContext ctx = ldapService_.getLdapContext();    
    String userDN = this.getDNFromUsername( m.getUserName());
    String groupDN = this.getGroupDNFromGroupId( m.getGroupId());
    
    String membershipDN = 
      ldapAttrMapping_.membershipTypeNameAttr + "=" + m.getMembershipType() + "," + groupDN;
    
    try {
      NameParser parser = ctx.getNameParser("");
      Name dn = parser.parse( membershipDN);
      Attributes attrs = ctx.getAttributes(dn);
      
      if (attrs != null) {       
        List members = this.getAttributes (attrs, ldapAttrMapping_.membershipTypeMemberValue);            
        if ( !members.contains( userDN)) {
          ModificationItem[] mods = new ModificationItem[1];
          mods[0] = new ModificationItem(
              DirContext.ADD_ATTRIBUTE,
              new BasicAttribute( ldapAttrMapping_.membershipTypeMemberValue, userDN));          
          try{
             preSave(m, true);   
             ctx.modifyAttributes(membershipDN, mods);
             postSave(m, true);
             return;
          }catch( Exception exp){  
            removeMembership( m.getGroupId(), true);
          }
        }
      }
    } catch (NameNotFoundException e) {          
    }  
    try{
      if(broadcast) preSave(m, true);      
      ctx.createSubcontext( membershipDN, ldapAttrMapping_.membershipToAttributes( m, userDN));     
      postSave(m, true);
    }catch( Exception exp){      
    }
  }
 
  public void linkMembership(User user, Group group, MembershipType mt, boolean broadcast) throws Exception{
    MembershipImpl membership = new MembershipImpl();
    membership.setMembershipType( mt.getName()) ;    
    membership.setUserName( user.getUserName());
    membership.setGroupId( group.getId());
    this.createMembership( membership, broadcast);
  }

  public Membership removeMembership(String id, boolean broadcast) throws Exception {  
    MembershipImpl m = new MembershipImpl();
    
    String membershipParts[] = id.split(",");
    if( membershipParts.length < 3) return null;
    String username = membershipParts[0];
    String membershipType = membershipParts[1];
    String groupId = membershipParts[2];
    
    m.setGroupId(groupId);
    m.setId(id);
    m.setMembershipType(membershipType);
    m.setUserName(username);
    
    LdapContext ctx = ldapService_.getLdapContext();    
    
    String userDN =  this.getDNFromUsername( username);     
    String groupDN = this.getGroupDNFromGroupId(groupId);
    String membershipDN =
      ldapAttrMapping_.membershipTypeNameAttr + "=" + membershipType + ", " + groupDN;   
    try {
      NameParser parser = ctx.getNameParser("");
      Name dn = parser.parse(membershipDN);
      Attributes attrs = ctx.getAttributes(dn);
      if (attrs == null) return m;
      // Group does exist, is userDN in it?      
      List<Object> members = 
           this.getAttributes(attrs, ldapAttrMapping_.membershipTypeMemberValue);
      boolean remove = false;
      for( int i=0; i<members.size(); i++){
        if( members.get( i).toString().trim().equals( userDN.trim())) {
          remove = true;
          break;
        }
      }
      if ( !remove) return m;     
      if( members.size() > 1) {
       ModificationItem[] mods = new ModificationItem[1];
       mods[0] = new ModificationItem( DirContext.REMOVE_ATTRIBUTE,
            new BasicAttribute( ldapAttrMapping_.membershipTypeMemberValue, userDN));
        if ( broadcast) preSave( m, true);
        ctx.modifyAttributes( membershipDN, mods);
        if ( broadcast)  postSave( m, true);    
      } else{
        if ( broadcast) preDelete(m);
        ctx.destroySubcontext(membershipDN);
        if ( broadcast)  postDelete(m);
      }
    } catch (NameNotFoundException e){
      e.printStackTrace();
    }   
    return m;
  }  

  public Membership findMembership(String id) throws Exception {
    String membershipParts[] = id.split(",");
    Membership membership = 
      findMembershipByUserGroupAndType(membershipParts[0], membershipParts[2], membershipParts[1]);
    return membership;
  }

  public Membership findMembershipByUserGroupAndType(String userName, String groupId, String type) throws Exception {
    MembershipImpl membership = new MembershipImpl();
    membership.setGroupId(groupId);
    membership.setUserName(userName);
    membership.setMembershipType(type);
    membership.setId(userName + "," + type + "," + groupId);
    
    LdapContext ctx = ldapService_.getLdapContext();
    String userDN = this.getDNFromUsername( userName);     
    String groupDN = getGroupDNFromGroupId(groupId);
    String membershipDN = ldapAttrMapping_.membershipTypeNameAttr + "=" + type + "," + groupDN;
   
    try {
      NameParser parser = ctx.getNameParser("");
      Name dn = parser.parse(membershipDN);
      Attributes attrs = ctx.getAttributes(dn);
      if (attrs == null) return null; 
      List<Object> members =
           this.getAttributes(attrs, ldapAttrMapping_.membershipTypeMemberValue);
      for( int i=0; i<members.size(); i++){
        if( members.get( i).toString().trim().equals( userDN.trim())) return membership;    
      }      
    } catch (NameNotFoundException e){        
    }
    return null;   
  }

  public Collection findMembershipsByUserAndGroup( String userName, String groupId) throws Exception {
    ArrayList<Membership> memberships = new ArrayList<Membership>();
    LdapContext ctx = ldapService_.getLdapContext();   
    String groupDN = this.getGroupDNFromGroupId( groupId);  

    SearchControls constraints = new SearchControls();
    constraints.setSearchScope( SearchControls.ONELEVEL_SCOPE);
    NamingEnumeration<SearchResult>  results = null;   
    
    try{       
       results = ctx.search( groupDN, "(objectclass=*)", constraints);
    }catch( Exception exp){        
    }
    
    if( results == null) return memberships;
    
     while( results.hasMore()) {
      SearchResult sr = results.next();
      NameParser parser = ctx.getNameParser("");        
      Name entryName = parser.parse( new CompositeName(sr.getName()).get(0));      
      String membershipDN = entryName + "," + groupDN;
      String type = this.explodeDN( parser.parse(membershipDN), true)[0];      
      Attributes attrs =  sr.getAttributes();
      Attribute attr = attrs.get( "member");
      if( attr != null){
        String temp;
        for( int i=0; i< attr.size(); i++){       
          temp = this.explodeDN( parser.parse( String.valueOf( attr.get(i))), true)[0];
          if( temp.trim().equals( userName)){
            MembershipImpl membership = new MembershipImpl();        
            membership.setId( userName + "," + type + "," + groupId);
            membership.setUserName(userName);
            membership.setMembershipType(type);
            membership.setGroupId( groupId);
            memberships.add( membership);
          }
        }
      }
    }
    return memberships;
  }

  public Collection findMembershipsByUser( String userName) throws Exception {   
    Collection<Membership> memberships = new ArrayList<Membership>();
    String userDN = "";
    try{
       userDN = this.getDNFromUsername(userName);
       if( userDN == null) return memberships;
    }catch( Exception exp){
      return memberships;
    }   
    String filter =  ldapAttrMapping_.membershipTypeMemberValue + "=" +this.escapeDN( userDN);  

    SearchControls constraints = new SearchControls();
    constraints.setSearchScope(SearchControls.SUBTREE_SCOPE);
    LdapContext ctx = ldapService_.getLdapContext();  
    NamingEnumeration<SearchResult> results = null;
    try{
      results = ctx.search( ldapAttrMapping_.groupsURL, filter, constraints);
    }catch( Exception exp){
      return memberships;
    }
    
    if( results == null) return memberships;
    
    while(results.hasMore()) {
      SearchResult sr = results.next();
      NameParser parser = ctx.getNameParser("");
      CompositeName name = new CompositeName(sr.getName());
      if (name.size() > 0){
        Name entryName = parser.parse(name.get(0));
        String membershipDN = entryName + "," + ldapAttrMapping_.groupsURL;        
        Group group = this.getGroupFromMembershipDN(membershipDN);
        String type = this.explodeDN(parser.parse(membershipDN), true)[0];
        MembershipImpl membership = new MembershipImpl();
        membership.setId(userName + "," + type + "," + group.getId());
        membership.setUserName(userName);
        membership.setMembershipType(type);
        membership.setGroupId(group.getId());
        memberships.add(membership);
      }
    }
    return memberships;
  }

  public Collection findMembershipsByGroup(Group group) throws Exception {
    ArrayList<Membership> memberships = new ArrayList<Membership>();
    LdapContext ctx = ldapService_.getLdapContext();   
    String groupDN = this.getGroupDNFromGroupId( group.getId());  

    SearchControls constraints = new SearchControls();
    constraints.setSearchScope( SearchControls.ONELEVEL_SCOPE);
    NamingEnumeration<SearchResult>  results = null;   
    
    try{       
       results = ctx.search( groupDN, "(objectclass=*)", constraints);
    }catch( Exception exp){        
    }
    
    if( results == null) return memberships;
    
     while( results.hasMore()) {
      SearchResult sr = results.next();
      NameParser parser = ctx.getNameParser("");        
      Name entryName = parser.parse( new CompositeName(sr.getName()).get(0));       
      String membershipDN = entryName + "," + groupDN;
      String type = this.explodeDN( parser.parse(membershipDN), true)[0];      
      Attributes attrs =  sr.getAttributes();
      Attribute attr = attrs.get( "member");
      String userName;
      for( int i=0; i<attr.size(); i++){
        userName = this.explodeDN( parser.parse( String.valueOf( attr.get(i))), true)[0];
        MembershipImpl membership = new MembershipImpl();        
        membership.setId( userName + "," + type + "," + group.getId());
        membership.setUserName(userName);
        membership.setMembershipType(type);
        membership.setGroupId( group.getId());
        memberships.add( membership);
      }
    }
    return memberships;
  }

  private void postDelete(Membership membership) throws Exception {   
    for( MembershipEventListener listener : listeners_ ) listener.postDelete(membership) ;  
  }
  
  private void preDelete(Membership membership) throws Exception {   
    for( MembershipEventListener listener : listeners_ ) listener.preDelete(membership) ;  
  }

  private void postSave(Membership membership, boolean isNew) throws Exception { 
    for( MembershipEventListener listener : listeners_ ) listener.postSave(membership, isNew) ;  
  }
  
  private void preSave(Membership membership, boolean isNew) throws Exception { 
    for( MembershipEventListener listener : listeners_ ) listener.preSave(membership, isNew) ;  
  }
}
