/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.organization.ldap;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.TreeMap;

import javax.naming.CompositeName;
import javax.naming.Name;
import javax.naming.NameNotFoundException;
import javax.naming.NameParser;
import javax.naming.NamingEnumeration;
import javax.naming.directory.Attributes;
import javax.naming.directory.BasicAttribute;
import javax.naming.directory.DirContext;
import javax.naming.directory.ModificationItem;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;
import javax.naming.ldap.Control;
import javax.naming.ldap.LdapContext;
import javax.naming.ldap.PagedResultsControl;
import javax.naming.ldap.PagedResultsResponseControl;
import javax.naming.ldap.SortControl;

import org.exoplatform.commons.utils.ObjectPageList;
import org.exoplatform.commons.utils.PageList;
import org.exoplatform.services.ldap.LDAPService;
import org.exoplatform.services.organization.Query;
import org.exoplatform.services.organization.User;
import org.exoplatform.services.organization.UserEventListener;
import org.exoplatform.services.organization.UserHandler;
import org.exoplatform.services.organization.impl.UserImpl;
/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Oct 14, 2005
 */
public class UserHandlerImpl extends BaseHandler implements  UserHandler {
  
  private List<UserEventListener> listeners_  = new ArrayList<UserEventListener>(5);
  
  public UserHandlerImpl(LDAPAttributeMapping ldapAttrMapping, LDAPService ldapService) {
    super(ldapAttrMapping, ldapService) ;
  }
  
  public void addUserEventListener(UserEventListener listener) {
    listeners_.add(listener) ;
  }
  
  public User createUserInstance() {  return new UserImpl() ;  }

  public void createUser(User user, boolean broadcast) throws Exception {   
    if(broadcast) preSave(user, true) ;
    LdapContext ctx = ldapService_.getLdapContext();
    String userDN = "cn="+ user.getUserName()+ ","+  ldapAttrMapping_.userURL;
    ctx.createSubcontext( userDN, ldapAttrMapping_.userToAttributes(user));
    if(broadcast) postSave(user, true) ;
  }
  
  public void saveUser(User user, boolean broadcast) throws Exception {
    if(broadcast) preSave(user, true)  ;
    LdapContext ctx = ldapService_.getLdapContext();
    String userDN = "cn="+ user.getUserName()+ ","+  ldapAttrMapping_.userURL;
    ctx.modifyAttributes(userDN, LdapContext.REPLACE_ATTRIBUTE, ldapAttrMapping_.userToAttributes(user));
    if(broadcast) postSave(user, true) ;
  } 

  public User removeUser(String userName, boolean broadcast) throws Exception {
    User user =  findUserByName(userName) ;
    if(user != null) {
      if(broadcast) preDelete(user)  ;
      LdapContext ctx = ldapService_.getLdapContext();
      String userDN = "cn="+  userName + ","+  ldapAttrMapping_.userURL;      
      removeUserFromMembership(ctx, userName);      
      ctx.destroySubcontext( userDN);      
      if(broadcast) postDelete(user)  ;
    }
    return user ;
  }
  
  private void removeUserFromMembership(LdapContext ctx, String userName) throws Exception {     
    String userDN = this.getDNFromUsername( userName);    
    String filter =  ldapAttrMapping_.membershipTypeMemberValue + "=" +this.escapeDN(userDN);  

    SearchControls constraints = new SearchControls();
    constraints.setSearchScope(SearchControls.SUBTREE_SCOPE);
    NamingEnumeration<SearchResult> results = ctx.search( ldapAttrMapping_.groupsURL, filter, constraints);
    
    while(results.hasMore()) {
      SearchResult sr = results.next();       
      try{
        Attributes attrs =  sr.getAttributes();   
        if( attrs.get( ldapAttrMapping_.membershipTypeMemberValue).size() > 1){          
          ModificationItem[] mods = new ModificationItem[1];
          mods[0] = new ModificationItem(
              DirContext.REMOVE_ATTRIBUTE,
              new BasicAttribute( 
                  ldapAttrMapping_.membershipTypeMemberValue, userDN));
          ctx.modifyAttributes( sr.getNameInNamespace(), mods); 
        }else
          ctx.destroySubcontext( sr.getNameInNamespace());        
      }catch( Exception exp){
        exp.printStackTrace();
      }
    }
  }

  public User findUserByName(String userName) throws Exception {
    User u  = this.findUserByName(userName, ldapAttrMapping_.userURL);
    if( u == null) u = this.findUserByNameInScope(userName, ldapAttrMapping_.baseURL);    
    return u;
  }
  
  public User findUserByName( String userName, String url) throws Exception{
    LdapContext ctx = ldapService_.getLdapContext();
    String userDN = "cn="+  userName + ","+  url;
    try{
      return ldapAttrMapping_.attributesToUser(this, ctx.getAttributes(userDN)) ;
    }catch( Exception exp){     
      return null;
    }
  }
  
  public User findUserByNameInScope( String userName, String url) throws Exception{    
    LdapContext ctx = ldapService_.getLdapContext();
    
    String filter = "cn="+userName;
    SearchControls constraints = new SearchControls();
    constraints.setSearchScope(SearchControls.SUBTREE_SCOPE);
    NamingEnumeration<SearchResult> results = ctx.search( url, filter, constraints);    
    
    while( results.hasMore()){
      SearchResult sr = results.next();     
      Attributes attrs = sr.getAttributes();      
      if( attrs != null ) {        
        Object pass = attrs.get("userPassword");
        if( pass != null) return ldapAttrMapping_.attributesToUser(this, attrs);        
      }
    }  
    return null;
  }
  

  public PageList findUsersByGroup(String groupId) throws Exception {    
    ArrayList<User> users = new ArrayList<User>();
    TreeMap< String, User> map = new TreeMap< String, User>();
    
    LdapContext ctx = ldapService_.getLdapContext();
    String groupDN = this.getGroupDNFromGroupId(groupId);
    String searchBase = groupDN;
    String filter = ldapAttrMapping_.membershipObjectClassFilter;
    
    SearchControls constraints = new SearchControls();
    constraints.setSearchScope(SearchControls.ONELEVEL_SCOPE);
    NamingEnumeration<SearchResult> results = ctx.search(searchBase, filter, constraints);
        
    while( results.hasMore()){
      SearchResult sr = results.next();
      Attributes attrs = sr.getAttributes();
      List<Object> members = this.getAttributes(attrs, ldapAttrMapping_.membershipTypeMemberValue);
      for (int x = 0; x < members.size(); x++){        
        User user = this.findUserByDN( (String)members.get(x), ctx);       
        if( user != null)  map.put(user.getUserName(), user);
      }
    }  
        
    for (Iterator<String> i = map.keySet().iterator(); i.hasNext();)   users.add( map.get(i.next()));    
    return new ObjectPageList(users, 20);
  }
  
  protected User findUserByDN (String userDN, LdapContext ctx) throws Exception {   
    if (userDN == null) return null;
    try {
      NameParser parser = ctx.getNameParser("");
      Name dn = parser.parse( userDN);
      Attributes attrs = ctx.getAttributes(dn);
      if (attrs == null) return null;
      User user = this.ldapAttrMapping_.attributesToUser( this, attrs);         
      user.setFullName( user.getFirstName()+" "+user.getLastName());  
      return user;
    } catch (NameNotFoundException e){     
      return null;
    }
  }

  public PageList getUserPageList(int pageSize) throws Exception {       
    List<User> users = new ArrayList<User>();
    NamingEnumeration<SearchResult> enumer;
    
    int itemCounter = pageSize;
    
    LdapContext ctx = ldapService_.getLdapContext();
    String searchBase = ldapAttrMapping_.userURL;
    String filter = ldapAttrMapping_.userObjectClassFilter;
    
    Control control[] = new Control[2];
    PagedResultsControl pagedResultsControl = new PagedResultsControl(pageSize, Control.CRITICAL);
    control[1] = pagedResultsControl;
    String keys[] = {ldapAttrMapping_.userUsernameAttr };
    SortControl sortControl = new SortControl(keys, false);
    control[0] = sortControl;
    
    ctx.setRequestControls(control);
    while (itemCounter >= pageSize) {
      itemCounter = 0;
      SearchControls constraints = new SearchControls();
      constraints.setSearchScope(SearchControls.SUBTREE_SCOPE);
      
      enumer = ctx.search(searchBase, filter, constraints);
      while (enumer.hasMore()) {
        itemCounter += 1;
        SearchResult result = enumer.next();
        Attributes attrs = result.getAttributes();
        // Create user object and add to arraylist          
        User user = this.ldapAttrMapping_.attributesToUser( this, attrs);         
        users.add(user);
      }
      if ( itemCounter >= pageSize) {
        Control[] responseControls = ctx.getResponseControls();
        for (int i = 0; i < responseControls.length; i++) {
          if (responseControls[i] instanceof PagedResultsResponseControl) {
            pagedResultsControl = new PagedResultsControl(
                pageSize, ((PagedResultsResponseControl) responseControls[i]).getCookie(), true);
            ctx.setRequestControls(new Control[] { pagedResultsControl });
          }
        }
      }     
    }     
    
    return new ObjectPageList(users, pageSize);
  }

  public PageList findUsers(Query q) throws Exception { 
    String filter = null;
    ArrayList<String> list = new ArrayList<String>();
    if (q.getUserName() != null && q.getUserName().length() > 0) {
      list.add("(" + ldapAttrMapping_.userUsernameAttr + "=" + q.getUserName() + ")");
    }
    if (q.getFirstName() != null && q.getFirstName().length() > 0) {
      list.add("(" + ldapAttrMapping_.userFirstNameAttr + "=" + q.getFirstName() + ")");
    }
    if (q.getLastName() != null && q.getLastName().length() > 0) {
      list.add("(" + ldapAttrMapping_.userLastNameAttr + "=" + q.getLastName() + ")");
    }
    if (q.getEmail() != null && q.getEmail().length() > 0) {
      list.add("(" + ldapAttrMapping_.userMailAttr + "=" + q.getEmail() + ")");
    }
    
    if (list.size() > 0){
      StringBuilder buffer = new StringBuilder();
      buffer.append("(&");
      if (list.size() > 1){
        for (int x = 0; x < list.size(); x++){
          if (x == (list.size() - 1)){
            buffer.append(list.get(x));
          } else {
            buffer.append(list.get(x)+" || ");
          }
        }
      } else {
        buffer.append(list.get(0));
      }
      buffer.append(" (" +ldapAttrMapping_.userObjectClassFilter + ") )");
      filter = buffer.toString();
    } else {
      filter = ldapAttrMapping_.userObjectClassFilter ;
    }
    String searchBase = ldapAttrMapping_.userURL ;
    return findUsers( searchBase, filter);
  }
  
  protected PageList findUsers(String searchBase,String filter) throws Exception {
    int pageSize = 100;
    long pageSleep = 0;    
    int itemCounter = pageSize;
    
    List<User> users = new ArrayList<User>();
    
    LdapContext ctx = ldapService_.getLdapContext();
    Control control[] = new Control[2];
    String keys[] = { ldapAttrMapping_.userUsernameAttr };
    SortControl sortControl = new SortControl(keys, false);
    control[0] = sortControl;
    PagedResultsControl pagedResultsControl =  new PagedResultsControl(pageSize, Control.CRITICAL) ;
    control[1] = pagedResultsControl;
    ctx.setRequestControls(control);
    
    do {
      itemCounter = 0;      
      SearchControls constraints = new SearchControls();
      constraints.setSearchScope(SearchControls.SUBTREE_SCOPE);
      
      NamingEnumeration<SearchResult> results =  ctx.search( searchBase, filter, constraints);      
      while (results != null && results.hasMore()) {
        itemCounter += 1;
        SearchResult entry = results.next();
        Attributes attrs = entry.getAttributes();
        // Create user object and add to arraylist
        User user = ldapAttrMapping_.attributesToUser(this, attrs);
        users.add(user);       
      }      
     
      if (itemCounter >= pageSize) {
        Control[] controls = ctx.getResponseControls();
        if (controls != null) {
          for (int i = 0; i < controls.length; i++) {
            if ( controls[i] instanceof PagedResultsResponseControl) {
              PagedResultsResponseControl prc = (PagedResultsResponseControl) controls[i];              
              pagedResultsControl =  new PagedResultsControl(pageSize, prc.getCookie(), true);
              ctx.setRequestControls(new Control[] { pagedResultsControl });
            }
          }
        }
      }      
      if (pageSleep > 0)  Thread.sleep(pageSleep);      
    } while (itemCounter >= pageSize );
    return new ObjectPageList(users, 20);
  }


  public boolean authenticate(String username, String password) throws Exception {
    User user = findUserByName( username);    
    if(user == null)  return false ; 
    return  user.getPassword().equals(password); 
  }
  
  protected void preSave(User user , boolean isNew) throws Exception {
    for (UserEventListener listener : listeners_)   listener.preSave(user, isNew) ;
  }
  
  protected void postSave(User user , boolean isNew) throws Exception {
    for (UserEventListener listener : listeners_) listener.postSave(user, isNew) ;
  }
  
  protected void preDelete(User user) throws Exception {
    for (UserEventListener listener : listeners_)   listener.preDelete(user) ;
  }
  
  protected void postDelete(User user) throws Exception {
    for (UserEventListener listener : listeners_)    listener.postDelete(user)  ;
  }
}
