package org.exoplatform.services.templates.velocity.impl;

import java.io.File;
import java.io.InputStream;
import java.lang.ref.SoftReference;
import java.net.URL;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections.ExtendedProperties;
import org.apache.velocity.Template;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;
import org.apache.velocity.runtime.resource.Resource;
import org.apache.velocity.runtime.resource.ResourceFactory;
import org.apache.velocity.util.StringUtils;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.services.templates.velocity.ResourceLoaderPlugin;
import org.exoplatform.services.templates.velocity.StaticResourceLoader;

public class StaticResourceLoaderImpl 
  extends ResourceLoaderPlugin implements StaticResourceLoader {
  
  private Map<String,String> templatesSearchPaths_ = new HashMap<String,String>();
  private Map templatePaths = new HashMap();
  private Map resourcesInfo_ = null ;
  private  boolean checkModified_ ;
  
  public StaticResourceLoaderImpl(InitParams params) throws Exception {
    List<String> paths = params.getValuesParam("velocity.template.path").getValues() ;
    for(String path : paths) templatesSearchPaths_.put(path, path) ;
    checkModified_ =  "true".equals(params.getValueParam("check.modified.resource").getValue()) ;
    resourcesInfo_ = new Hashtable();
  }
  
  public void init(ExtendedProperties configuration) {
  }
  
  public synchronized InputStream getResourceStream(String templateName) throws ResourceNotFoundException {
    ResourceInfo rinfo = (ResourceInfo)resourcesInfo_.get(templateName);
    try  {
      return rinfo.url.openStream() ;
    } catch (Exception ex) {
      ex.printStackTrace() ;
    }
    return null ;
  }
  
  public synchronized URL getURL(String templateName) throws ResourceNotFoundException {
    String template = StringUtils.normalizePath(templateName);
    if ( template == null || template.length() == 0 ) {
      throw new ResourceNotFoundException ("Template parameter cannot be null or empty");
    }
    
    if (template.startsWith("/")) template = template.substring(1);
    for (String path : templatesSearchPaths_.values()) {
      URL url = findURL(path, template);
      if (url != null)  return url;        
    }
    //Try the jar file so 
    ClassLoader cl = Thread.currentThread().getContextClassLoader() ;
    URL url = cl.getResource(templateName) ;
    if(url != null)  return url ;
    throw new ResourceNotFoundException( "FileResourceLoader Error: cannot find resource " + template);
  }
  
  private URL findURL(String path, String template) {
    try  {
      File file = new File( path, template );   
      //System.err.println("check  file : " + file.getAbsolutePath()) ;
      if ( file.canRead()) return file.toURL() ;
    } catch(Exception ex){
      ex.printStackTrace() ;
    }
    return null;
  }
  
  
  public boolean isSourceModified(Resource resource) {
    boolean modified = true;   
    String fileName = resource.getName();
    String path = (String) templatePaths.get(fileName);
    File currentFile = null;
    
    for (int i = 0; currentFile == null && i < templatesSearchPaths_.size(); i++) {
      String testPath = (String) templatesSearchPaths_.get(i);
      File testFile = new File(testPath, fileName);
      if (testFile.canRead())
      {
        currentFile = testFile;
      }
    }
    File file = new File(path, fileName);
    if (currentFile == null || !file.exists()) {
     
    } else if (currentFile.equals(file) && file.canRead()) {
      modified = (file.lastModified() != resource.getLastModified());
    }
    return modified;
  }
  
  public long getLastModified(Resource resource) {
    String path = (String) templatePaths.get(resource.getName());
    File file = new File(path, resource.getName());
    if (file.canRead()){
      return file.lastModified();
    }
    return 0;       
  }
  
  public void addPath(String newPath) {
    templatesSearchPaths_.put(newPath, newPath) ;
  }
  
  public Template getTemplate(String name)  throws  Exception  {
    ResourceInfo rinfo = (ResourceInfo)resourcesInfo_.get(name);
    if(rinfo != null) {
      if(rinfo.ref != null) {
        Resource resource  =  rinfo.ref.get();
        if(resource != null) return (Template) resource;
      }
    }
    return null ;
  }
  
  public Resource getResource(String resourceName, int resourceType, String encoding )
  throws ResourceNotFoundException, ParseErrorException, Exception {
    ResourceInfo rinfo = (ResourceInfo)resourcesInfo_.get(resourceName);
    if(rinfo == null) {
      URL url =  getURL(resourceName) ;
      rinfo = new ResourceInfo(resourceName, url) ;
      resourcesInfo_.put(resourceName, rinfo) ;
    }
    Resource resource  = null ;
    if(rinfo.ref != null) resource =  rinfo.ref.get();
    if(resource != null) {
      if(checkModified_){
        refreshResource(resource, encoding);
      }
      return resource ;
    }
    resource = ResourceFactory.getResource(resourceName, resourceType);  
    resource.setRuntimeServices( rsvc );
    resource.setName( resourceName );
    resource.setEncoding( encoding );
    resource.setResourceLoader(this);
    resource.process() ;
    if (resource.getData() == null) {
      throw new ResourceNotFoundException("Unable to find resource '" + resourceName + "'");
    }
    resource.setLastModified(0);//long howOldItWas = 0;  // Initialize to avoid warning  
    resource.setModificationCheckInterval(getModificationCheckInterval());
    resource.touch();
    rinfo.ref = new SoftReference<Resource>(resource) ;
    return resource;            
  }
  
  protected void refreshResource( Resource resource, String encoding )
  throws ResourceNotFoundException, ParseErrorException, Exception {
    resource.touch();
    if(  resource.isSourceModified() ) {
      if (!resource.getEncoding().equals( encoding )) {
        rsvc.error("Declared encoding for template '" + resource.getName() 
            + "' is different on reload.  Old = '" + resource.getEncoding()
            + "'  New = '" + encoding );
        resource.setEncoding( encoding );
      }
      
      long howOldItWas = resource.getResourceLoader().getLastModified( resource );
      resource.process();
      resource.setLastModified( howOldItWas );
    }
  }
  
  public boolean canLoadResource(String s)   { return true ; }

  public void addTemplate(String name, URL url) {
    resourcesInfo_.put(name, new ResourceInfo(name, url)) ;
  }
  
  static public class ResourceInfo   {
    String name ;
    URL url ;
    SoftReference<Resource> ref ;
    
    public  ResourceInfo(String name, URL url) {
      this.name = name ;
      this.url =  url ;
    }
  }
}
