package org.exoplatform.faces.core.renderer.html;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import javax.portlet.ActionResponse;
import javax.portlet.PortletRequest;
import javax.portlet.WindowState;
import javax.portlet.WindowStateException;

import org.exoplatform.commons.utils.ExpressionUtil;
import org.exoplatform.faces.FacesConstants;
import org.exoplatform.faces.core.component.Node;
import org.exoplatform.faces.core.component.UIExoComponent;
import org.exoplatform.faces.core.component.model.Parameter;

public class ListNodesRenderer extends NodeMenuRenderer {

  public void decode(FacesContext context, UIComponent component) {
    super.decode(context, component);
    Map paramMap = context.getExternalContext().getRequestParameterMap();
    String action = (String) paramMap.get(FacesConstants.ACTION);
    try {
      ExternalContext eContext = FacesContext.getCurrentInstance()
        .getExternalContext();
      ActionResponse aResponse = (ActionResponse) eContext.getResponse();
      if ("selectTab".equals(action)) {
        aResponse.setWindowState(WindowState.MAXIMIZED);
      } else if (CANCEL_ACTION.equals(action)) {
        aResponse.setWindowState(WindowState.NORMAL);
      }
    } catch (WindowStateException e) {
      e.printStackTrace();
    }
  }

  final public void encodeChildren(FacesContext context, UIComponent component)
      throws IOException {
    ResponseWriter w = context.getResponseWriter();
    ResourceBundle res = getApplicationResourceBundle(context
        .getExternalContext());
    UIExoComponent uiComponent = (UIExoComponent) component;
    List children = uiComponent.getChildren();
    if (children.size() == 0)
      return;
    UIExoComponent uiSelectTab = null;

    String clazz = uiComponent.getClazz();
    if (clazz == null) {
      w.write("<div>");
    } else {
      w.write("<div class='");
      w.write(clazz);
      w.write("'>");
    }
    if (children.size() > 0) {
      if (isMaximized()) {
        for (int i = 0; i < children.size(); i++) {
          UIExoComponent uiChild = (UIExoComponent) children.get(i);
          if (uiChild.isRendered())
            uiSelectTab = uiChild;
        }
        w.write("<div class='");
        w.write(getBodyCssClass());
        w.write("'>");
        uiSelectTab.encodeBegin(context);
        uiSelectTab.encodeChildren(context);
        uiSelectTab.encodeEnd(context);
        Parameter cancelParam = new Parameter(ACTION, CANCEL_ACTION);
        Parameter[] params = { cancelParam };
        String label = res.getString("UINode.label.cancel");
        linkRenderer_.render(w, uiComponent, label, params);
        w.write("</div>");
      } else {
        Parameter tabIdParam = new Parameter("tabId", "");
        Parameter[] params = { SELECT_TAB, tabIdParam };
        w.write("<ul class='");
        w.write(getCssClass());
        w.write("'>");
        for (int i = 0; i < children.size(); i++) {
          UIExoComponent uiChild = (UIExoComponent) children.get(i);
          tabIdParam.setValue(uiChild.getId());
          String label = null;
          if (uiChild instanceof Node) {
            Node uiNode = (Node) uiChild;
            label = uiNode.getName();
          }
          if (label == null) {
            label = ExpressionUtil.getValue(res, "UINode.label."
                + uiChild.getId());
          }
          w.write("<li>");
          linkRenderer_.render(w, uiComponent, label, params);
          w.write("</li>");
        }
        w.write("</ul>");
      }
    } else {
      w.write(res.getString("UINode.msg.no-node"));
    }
    w.write("</div>");
  }

  protected String getCssClass() {
    return "UIListNodes";
  }

  protected String getBodyCssClass() {
    return "UIBodyListNodes";
  }

  private boolean isMaximized() {
    ExternalContext eContext = FacesContext.getCurrentInstance()
        .getExternalContext();
    PortletRequest pRequest = (PortletRequest) eContext.getRequest();
    WindowState wState = pRequest.getWindowState();
    if (WindowState.MAXIMIZED.equals(wState))
      return true;
    return false;
  }

}
