/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.faces.user.component;

import java.util.Collection;

import org.exoplatform.Constants;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.organization.Group;
import org.exoplatform.services.organization.OrganizationService;
/**
 * Wed, Dec 22, 2003 @ 23:14 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UISelectBox.java,v 1.6 2004/06/26 03:47:31 tuan08 Exp $
 */
public class UIExoPermissionBrowser extends UIExoCommand implements Selector {
  static String CHANGE_GROUP = "changeGroup" ;
  static String SELECT_GROUP = "selectGroup" ;
  static String SELECT_MEMBERSHIP = "selectMembershipType" ;
  static String SELECT_PERMISSION = "selectPermission" ;
  
  static Parameter[] CHANGE_GROUP_PARAMS = {new Parameter(ACTION , CHANGE_GROUP) };
  static Parameter[] SELECT_GROUP_PARAMS = {new Parameter(ACTION , SELECT_GROUP) };
  static Parameter[] SELECT_PERMISSION_PARAMS = {new Parameter(ACTION , SELECT_PERMISSION) };
  static Parameter[] CANCEL_PARAMS = {new Parameter(ACTION , CANCEL_ACTION) };
  static Parameter[] SELECT_MEMBERSHIP_PARAMS = {new Parameter(ACTION , SELECT_MEMBERSHIP) };
  
  private Group selectGroup_ ;
  private Group parentGroup_ ;
  private Collection selectChildrenGroup_ ;
  private Collection parentChildrenGroup_ ;
  private String selectMembership_ = "*";
  private String selectGroupId_ ;
  private Collection membershipTypes_ ;
  private SelectorHandler handler_ ;
  private String[] parameters_;
  
  public UIExoPermissionBrowser() throws Exception {
    setRendererType("VelocityRenderer") ;
    setId("UIExoPermissionBrowser");
    OrganizationService service = 
      (OrganizationService) PortalContainer.getComponent(OrganizationService.class);
    membershipTypes_ =  service.getMembershipTypeHandler().findMembershipTypes() ;
    addActionListener(SelectMembershipActionListener.class, SELECT_MEMBERSHIP) ;
    addActionListener(ChangeGroupActionListener.class, CHANGE_GROUP) ;
    addActionListener(SelectGroupActionListener.class, SELECT_GROUP) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
    addActionListener(SelectPermissionActionListener.class, SELECT_PERMISSION) ;
    changeGroup(null) ;
  } 
  
  public void setHandler(SelectorHandler handler) {
    handler_ = handler ;
  }
  
  public Parameter[] getChangeGroupParams() { return CHANGE_GROUP_PARAMS ; }
  public Parameter[] getSelectGroupParams() { return SELECT_GROUP_PARAMS ; }
  public Parameter[] getCancelParams() { return CANCEL_PARAMS ; }
  public Parameter[] getSelectMembershipParams() { return SELECT_MEMBERSHIP_PARAMS ; }
  public Parameter[] getSelectPermissionParams() { return SELECT_PERMISSION_PARAMS ; }
  
  public Group getSelectGroup() { return selectGroup_ ; }
  
  public Group getParentGroup() { return parentGroup_ ; }
  
  public String getParentGroupId() {
    if(parentGroup_ == null) return "" ;
    return parentGroup_.getId() ; 
  }
  
  public String getSelectMembership() { return selectMembership_ ; }
  public String getSelectGroupId() { return selectGroupId_ ; }
  
  public void setParameters(String[] args) {
    parameters_ = args;
  }
  
  public String[] getParameters(){
    return parameters_;
  }

  public String getInputValue() {
    return selectMembership_ + ":" + selectGroupId_;
  }  
  
  public Collection getSelectChildrenGroup() { return selectChildrenGroup_ ; }
  public Collection getParentChildrenGroup() { return parentChildrenGroup_ ; }
  public Collection getMembershipTypes() { return membershipTypes_ ; }
  public String getObjectIds( String id1, String id2){
    return id1+":"+id2 ;
  }
  public void changeGroup(String groupId) throws Exception {
    OrganizationService service = 
      (OrganizationService) PortalContainer.getComponent(OrganizationService.class);
    if(groupId != null && groupId.length() > 0) {
      selectGroup_  = service.getGroupHandler().findGroupById(groupId) ;
      if(selectGroup_.getParentId() != null) {
        parentGroup_ = service.getGroupHandler().findGroupById(selectGroup_.getParentId()) ;
        parentChildrenGroup_ = service.getGroupHandler().findGroups(parentGroup_) ;
      } else {
        parentGroup_ = null ; 
        parentChildrenGroup_ = Constants.EMPTY_LIST ;
      }
      selectChildrenGroup_ = service.getGroupHandler().findGroups(selectGroup_) ;
    } else {
      selectGroup_  = null ;
      parentChildrenGroup_ = service.getGroupHandler().findGroups(null) ;
      selectChildrenGroup_ = parentChildrenGroup_ ;
      parentGroup_ = null  ;
    }
  }
  
  static public class ChangeGroupActionListener extends ExoActionListener {
	  public void execute(ExoActionEvent event) throws Exception {
      UIExoPermissionBrowser uiComp = (UIExoPermissionBrowser) event.getSource() ;
		  String groupId = event.getParameter(OBJECTID) ;
      uiComp.changeGroup(groupId) ;
	  }
  }
  
  static public class SelectGroupActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIExoPermissionBrowser uiComp = (UIExoPermissionBrowser) event.getSource() ;
      uiComp.selectGroupId_ = event.getParameter(OBJECTID) ;
      uiComp.handler_.select(uiComp) ;
    }
  }
  static public class SelectPermissionActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIExoPermissionBrowser uiComp = (UIExoPermissionBrowser) event.getSource() ;
      String objectIds = event.getParameter(OBJECTID) ;
      String memberShipId = objectIds.substring(0, objectIds.indexOf(":")) ;
      String groupId = objectIds.substring(objectIds.indexOf(":") + 1, objectIds.length()) ;
      uiComp.selectMembership_ = memberShipId ;
      uiComp.selectGroupId_ = groupId ;
      uiComp.handler_.select(uiComp) ;
      uiComp.selectMembership_ = "*" ;
    }
  }
  static public class SelectMembershipActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIExoPermissionBrowser uiComp = (UIExoPermissionBrowser) event.getSource() ;
      uiComp.selectMembership_ = event.getParameter(OBJECTID) ; 
    }
  }
  
 static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIExoPermissionBrowser uiComp = (UIExoPermissionBrowser) event.getSource() ;
      uiComp.handler_.cancel(uiComp) ;
    }
  }

}