/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.faces.component;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.event.PhaseId;
import javax.faces.validator.Validator;
import javax.faces.validator.ValidatorException;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.core.validator.EmptyFieldValidator;
import org.exoplatform.faces.core.validator.IdentifierValidator;
import org.exoplatform.faces.user.component.UIExoPermissionBrowser;
import org.exoplatform.faces.user.component.UIExoPermissionSelector;
import org.exoplatform.portal.Information;
import org.exoplatform.portal.session.ExoPortal;
import org.exoplatform.services.portal.PortalConfigService;
import org.exoplatform.services.portal.model.Node;
import org.exoplatform.services.portal.model.PageReference;
/**
 * Sat, Jan 03, 2004 @ 11:16
 * @author: Tuan Nguyen
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UIPageNodeForm.java,v 1.19 2004/09/29 17:44:51 benjmestrallet Exp $
 */
public class UIPageNodeForm extends UISimpleForm implements SelectorHandler {
  static String  SEARCH_XHTML_REF_PAGE   = 
    org.exoplatform.faces.core.Util.encodeActionPhase("searchXhtmlRefPage", PhaseId.APPLY_REQUEST_VALUES) ;
  static String  SEARCH_XHTMLMP_REF_PAGE = 
    org.exoplatform.faces.core.Util.encodeActionPhase("searchXhtmlmpRefPage", PhaseId.APPLY_REQUEST_VALUES) ;
  static String  SEARCH_GROUP_FOR_VIEW_PERMISSION = 
    org.exoplatform.faces.core.Util.encodeActionPhase("searchGroupForViewPerm", PhaseId.APPLY_REQUEST_VALUES) ;
  static String  SEARCH_GROUP_FOR_EDIT_PERMISSION = 
    org.exoplatform.faces.core.Util.encodeActionPhase("searchGroupForEditPerm", PhaseId.APPLY_REQUEST_VALUES) ;
  
  static String NAME = "name" , DISPLAY_NAME = "displayName", 
                VIEW_PERMISSION = "viewPermission", EDIT_PERMISSION = "editPermission",
                ICON = "icon", XHTML_REFERENCE = "xhtmlPageReference" ,
                XHTMLMP_REFERENCE = "xhtmlmpPageReference" , XHTML_VISIBLE = "xhtmlVisible",
                XHTMLMP_VISIBLE = "xhtmlmpVisible" , DESCRIPTION = "description" ;
  
  Node                selectNode_;
  Node                parentNode_;
  PortalConfigService service_;
  String              lastSearchAction_;

  public UIPageNodeForm(PortalConfigService service) throws Exception {
    super("pageNodeForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer") ;
    service_ = service;

    add(new UIStringInput(NAME, "").addValidator(IdentifierValidator.class)) ;
    add(new UIStringInput(DISPLAY_NAME, "").addValidator(EmptyFieldValidator.class));
    add(new UIStringInput(ICON, "")) ;
    UIExoPermissionSelector permissionSelector =   
      new UIExoPermissionSelector(VIEW_PERMISSION, null) ;
    add(permissionSelector) ;
    permissionSelector =  
      new UIExoPermissionSelector(EDIT_PERMISSION, null) ;
    add(permissionSelector) ;
    add(new UITextArea(DESCRIPTION, "")) ;
    add(new UIStringInput(XHTML_REFERENCE, "").addValidator(EmptyFieldValidator.class)) ;
    add(new UICheckBox(XHTML_VISIBLE, "true")) ;
    add(new UIStringInput(XHTMLMP_REFERENCE, "")) ;
    add(new UICheckBox(XHTMLMP_VISIBLE, "true")) ;
    addValidator(NodeNameValidator.class) ;
    addActionListener(SaveActionListener.class, SAVE_ACTION) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
    addActionListener(SearchPageActionListener.class, SEARCH_XHTML_REF_PAGE) ;
    addActionListener(SearchPageActionListener.class, SEARCH_XHTMLMP_REF_PAGE) ;
    addActionListener(SearchGroupActionListener.class, SEARCH_GROUP_FOR_VIEW_PERMISSION) ;
    addActionListener(SearchGroupActionListener.class, SEARCH_GROUP_FOR_EDIT_PERMISSION) ;
  }

  
  public String getNodeCreator() {
    if(selectNode_ == null)  return null ;
    String creator = selectNode_.getCreator() ;
    if(creator == null)  return "system" ; 
    return creator  ;  
  }
  
  public String getNodeModifier() {  
    if(selectNode_ == null)  return null ;
    String modifier = selectNode_.getModifier() ;
    if(modifier == null)  return "system" ; 
    return modifier  ;  
  }
  
  public void  setEditingNavigationNode( Node parent, Node node) {
    selectNode_ = node ;
    parentNode_ = parent ;
    if (node != null) {
      getUIStringInput(NAME).setValue(node.getName()) ;
      getUIStringInput(NAME).setEditable(false) ;
      getUIStringInput(DISPLAY_NAME).setValue(node.getLabel()) ;
      ((UIExoPermissionSelector)getUIInput(VIEW_PERMISSION)).setPermission(node.getViewPermission()) ;
      ((UIExoPermissionSelector)getUIInput(EDIT_PERMISSION)).setPermission(node.getEditPermission()) ;
      getUIStringInput(ICON).setValue("") ;
      getUIStringInput(DESCRIPTION).setValue("") ;
      PageReference xhtmlPageRef = node.getPageReference(ExoPortal.XHTML_MIME_TYPE);
      getUIStringInput(XHTML_REFERENCE).setValue(xhtmlPageRef.getPageReference()) ;
      getUICheckBox(XHTML_VISIBLE).setSelect(xhtmlPageRef.isVisible()) ;
      PageReference xhtmlmpPageRef = node.getPageReference(ExoPortal.XHTMLMP_MIME_TYPE) ;
      if(xhtmlmpPageRef == null) {
        getUIStringInput(XHTMLMP_REFERENCE).setValue("") ;
        getUICheckBox(XHTMLMP_VISIBLE).setSelect(false) ;
      } else {
        getUIStringInput(XHTMLMP_REFERENCE).setValue(xhtmlmpPageRef.getPageReference()) ;
        getUICheckBox(XHTMLMP_VISIBLE).setSelect(xhtmlmpPageRef.isVisible()) ;
      }
    } else {
      getUIStringInput(NAME).setValue("") ;
      getUIStringInput(NAME).setEditable(true) ;
      getUIStringInput(DISPLAY_NAME).setValue("");
      getUIInput(VIEW_PERMISSION).reset() ;
      getUIInput(EDIT_PERMISSION).reset();
      getUIStringInput(ICON).setValue("");
      getUIStringInput(DESCRIPTION).setValue("");
      getUIStringInput(XHTML_REFERENCE).setValue("");
      getUICheckBox(XHTML_VISIBLE).setSelect(false);
      getUIStringInput(XHTMLMP_REFERENCE).setValue("");
      getUICheckBox(XHTMLMP_VISIBLE).setSelect(false);
    }
  }

  public void returnReferencePage(String refPage) {
    if (lastSearchAction_.equals(SEARCH_XHTMLMP_REF_PAGE)) {
      getUIStringInput(XHTML_REFERENCE).setValue(refPage);
    } else {
      getUIStringInput(XHTML_REFERENCE).setValue(refPage);
    }
  }

  public String getSearchXHTMLPage() { return SEARCH_XHTML_REF_PAGE ; }
  public String getSearchXHTMLMPPage() { return SEARCH_XHTMLMP_REF_PAGE ; }
  public String getSearchGroupForViewPerm() { return SEARCH_GROUP_FOR_VIEW_PERMISSION; }
  public String getSearchGroupForEditPerm() { return SEARCH_GROUP_FOR_EDIT_PERMISSION; }
  
  public String getURI() {
    if(selectNode_ == null) return "" ;
    return selectNode_.getUri() ;
  }
  
  public void save() throws Exception {
    UIPortal uiPortal = (UIPortal) getAncestorOfType(UIPortal.class);
    if (selectNode_ == null) {
      String parentURI = parentNode_.getUri();
      String name = getUIStringInput(NAME).getValue() ;
      if ("/".equals(parentURI))  parentURI = "";
      selectNode_ = service_.createNodeInstance();
      selectNode_.setUri(parentURI + "/" + name);
      selectNode_.setName(name);
      PageReference pref = new PageReference();
      pref.setType(ExoPortal.XHTML_MIME_TYPE);
      pref.setVisible(true);
      selectNode_.getPageReference().add(pref);
      parentNode_.addChild(selectNode_);
    }
    selectNode_.setLabel(getUIStringInput(DISPLAY_NAME).getValue());
    selectNode_.setViewPermission(((UIExoPermissionSelector)getUIInput(VIEW_PERMISSION)).getPermission()) ;
    selectNode_.setEditPermission(((UIExoPermissionSelector)getUIInput(EDIT_PERMISSION)).getPermission()) ;
    selectNode_.setModifier(uiPortal.getRemoteUser()) ;
    if(selectNode_.getCreator() == null) selectNode_.setModifier(uiPortal.getRemoteUser()) ;
    PageReference pageRef = selectNode_.getPageReference(ExoPortal.XHTML_MIME_TYPE);
    pageRef.setPageReference(getUIStringInput(XHTML_REFERENCE).getValue());
    pageRef.setVisible(getUICheckBox(XHTML_VISIBLE).getSelect());
    String xhtmlmpReference =getUIStringInput(XHTMLMP_REFERENCE).getValue();
    if (xhtmlmpReference == null || xhtmlmpReference.length() == 0) {
      selectNode_.removePageReference(ExoPortal.XHTMLMP_MIME_TYPE);
    } else {
      pageRef = selectNode_.getPageReference(ExoPortal.XHTMLMP_MIME_TYPE);
      if (pageRef == null) {
        pageRef = new PageReference();
        pageRef.setType(ExoPortal.XHTMLMP_MIME_TYPE);
        selectNode_.getPageReference().add(pageRef);
      }
      pageRef.setPageReference(xhtmlmpReference);
      pageRef.setVisible(getUICheckBox(XHTMLMP_VISIBLE).getSelect());
    }
    uiPortal.setComponentModified(true);
    UINavigationManager uiNav = 
      (UINavigationManager) PortalComponentCache.findPortalComponent(UINavigationManager.class) ;
    uiPortal.setBodyComponent(uiNav);
  }
  
  public void cancel(Selector selector) {
    UIPortal uiPortal = (UIPortal) selector.getAncestorOfType(UIPortal.class);
    uiPortal.setBodyComponent(this);
  }

  public void select(Selector selector) throws Exception {
    String[] parameters = selector.getParameters();
    if(parameters[0].equals(SEARCH_GROUP_FOR_VIEW_PERMISSION)) {    
      UIExoPermissionSelector permSelector = (UIExoPermissionSelector)getUIInput(VIEW_PERMISSION) ;
      permSelector.setPermission(selector.getInputValue()) ;
    } else {
      UIExoPermissionSelector permSelector = (UIExoPermissionSelector)getUIInput(EDIT_PERMISSION) ;
      permSelector.setPermission(selector.getInputValue()) ;
    }
    UIPortal uiPortal = (UIPortal) selector.getAncestorOfType(UIPortal.class);
    uiPortal.setBodyComponent(this);
  }
  
  static public class SearchPageActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPageNodeForm uiForm = (UIPageNodeForm) event.getComponent();
      String action = event.getAction();
      uiForm.lastSearchAction_ = action;
      UIPortal uiPortal = (UIPortal) uiForm.getAncestorOfType(UIPortal.class);
      UIPageBrowser uiPageBrowser = 
        (UIPageBrowser) PortalComponentCache.findPortalComponent(UIPageBrowser.class);
      if (uiPageBrowser != null) {
        uiPageBrowser.setReturnModule(UIPageBrowser.UI_NAVIGATION_NODE_FORM);
        uiPortal.setBodyComponent(uiPageBrowser);
      }
    }
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPageNodeForm uiForm = (UIPageNodeForm) event.getComponent();
      uiForm.save();
    }
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPageNodeForm uiForm = (UIPageNodeForm) event.getComponent();
      UIPortal uiPortal = (UIPortal) uiForm .getAncestorOfType(UIPortal.class);
      UINavigationManager uiNav = 
        (UINavigationManager) PortalComponentCache.findPortalComponent(UINavigationManager.class) ;
      uiPortal.setBodyComponent(uiNav);
    }
  }
  
  static public class SearchGroupActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPageNodeForm uiForm = (UIPageNodeForm) event.getComponent();
      UIPortal uiPortal = (UIPortal) uiForm.getAncestorOfType(UIPortal.class);
      UIExoPermissionBrowser uiGroupBrowser = 
        (UIExoPermissionBrowser)PortalComponentCache.findPortalComponent(UIExoPermissionBrowser.class);
      uiGroupBrowser.setHandler(uiForm) ;
      String[] params = {event.getAction()};
      uiGroupBrowser.setParameters(params) ;
      uiPortal.setBodyComponent(uiGroupBrowser) ;
    }
  }
 
  static public class NodeNameValidator implements Validator {
    public void validate(FacesContext context, UIComponent component,
                         Object value) throws ValidatorException {
      UIPageNodeForm uiForm = (UIPageNodeForm) component;
      if (uiForm.selectNode_ == null) {
        UIPortal uiPortal = (UIPortal) uiForm.getAncestorOfType(UIPortal.class);
        Node currentNode = uiPortal.getSelectedNode();
        if (currentNode.hasChild(uiForm.getUIStringInput(NAME).getValue())) {
          throw new ValidatorException(new ExoFacesMessage("#{UIPageNodeForm.msg.node-name-exist}"));
        }
      }
    }
  }
}