/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.faces.component;

import java.util.List;
import java.util.ResourceBundle;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.container.client.http.HttpClientInfo;
import org.exoplatform.faces.core.component.ComponentTemplate;
import org.exoplatform.faces.core.component.InformationProvider;
import org.exoplatform.portal.Information;
import org.exoplatform.portal.PortalConstants;
import org.exoplatform.portal.faces.component.logic.PortalComponentSecurity;
import org.exoplatform.portal.faces.component.logic.PortalLocale;
import org.exoplatform.portal.faces.component.logic.PortalMode;
import org.exoplatform.portal.faces.component.logic.PortalSkin;
import org.exoplatform.portal.faces.listener.page.PageActionListener;
import org.exoplatform.portal.faces.listener.portal.NodeActionListener;
import org.exoplatform.portal.faces.listener.portal.PortalActionListener;
import org.exoplatform.portal.session.ExoPortal;
import org.exoplatform.portal.session.RequestInfo;
import org.exoplatform.services.portal.PortalConfigService;
import org.exoplatform.services.portal.model.Component;
import org.exoplatform.services.portal.model.Container;
import org.exoplatform.services.portal.model.Node;
import org.exoplatform.services.portal.model.NodeVisitor;
import org.exoplatform.services.portal.model.PortalConfig;
import org.exoplatform.services.portal.skin.SkinConfigService;
import org.exoplatform.services.portal.skin.model.Style;
import org.exoplatform.services.portletcontainer.pci.ExoWindowID;

/**
 * Date: Aug 11, 2003
 * 
 * @author : Mestrallet Benjamin
 * @email: benjmestrallet@users.sourceforge.net
 * @author: Tuan Nguyen
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UIPortal.java,v 1.36 2004/10/27 02:53:05 tuan08 Exp $
 */
public class UIPortal extends UIPortalComponent 
  implements InformationProvider, ExoPortal, ComponentTemplate {

  private String             userCss_;
  private Node               root_;
  private Node               selectNode_;
  private PortalConfig       componentModel_;
  private boolean            dirty_                = false;
  private Information        information_    = new Information();
  private String             portalOwner_;
  private boolean            queueEvent_;
  private int                portalMode_;
  private boolean            editPortalCapability_;
  private boolean            editNavigationCapability_;

  public UIPortal(PortalConfigService configService, RequestInfo rinfo) throws Exception {
    portalOwner_ = rinfo.getPortalOwner();
    componentModel_ = configService.getPortalConfig(portalOwner_);
    if(componentModel_.getId() == null) componentModel_ .setId("portal");
    setBasicProperties(componentModel_, "default");
    HttpClientInfo clientInfo = (HttpClientInfo) SessionContainer.getInstance().getClientInfo();

    root_ = configService.getNodeNavigation(portalOwner_);
    root_.setVisible(configService.getPortalACL(), rinfo.getPortalOwner(), clientInfo.getRemoteUser(), true);
    selectNode_ = root_.findNode(rinfo.getPageName());
    if (selectNode_ == null)
      selectNode_ = root_;
    UIContainer uiPortalLayout = null;
    if (ExoPortal.XHTMLMP_MIME_TYPE.equals(clientInfo.getPreferredMimeType())) {
      uiPortalLayout = new UIContainer(componentModel_.getMobilePortalLayout(), "default",
                                       ExoWindowID.MOBILE_PORTAL_CONFIG);
    } else {
      uiPortalLayout = new UIContainer(componentModel_.getLayout(), "default",
                                       ExoWindowID.DEFAULT_PORTAL_CONFIG);
    }
    if ("exotest".equals(portalOwner_)) {
      initTestPortal(configService);
    }
    getChildren().add(uiPortalLayout);  
    PortalLocale.changeLocale(this);
    new PortalComponentSecurity(configService).setCapabilities(this);    
    findUIBody().initDefaultBodyComponent();
    userCss_ = new PortalSkin().getCSS(this);
    setId("portal");
    registerListeners();
  }

  public void initTestPortal(PortalConfigService configService) {
    PortalComponentCache cache = (PortalComponentCache) SessionContainer
        .getComponent(PortalComponentCache.class);
    UIPage uiPage = cache.getUIPage(root_, this);
    try {
      FacesContext context = FacesContext.getCurrentInstance();
      HttpServletRequest request = (HttpServletRequest) context.getExternalContext().getRequest();
      String[] portletNames = request.getParameterValues("portletName");
      String style = request.getParameter("style");
      if (style == null)
        style = "default";
      if (portletNames != null) {
        List children = uiPage.getChildren();
        children.clear();
        for (int i = 0; i < portletNames.length; i++) {
          String[] keys = StringUtils.split(portletNames[i], "/");
          String portletApp = keys[0];
          String portletName = keys[1];
          String id = "window-" + portletName;
          if (keys.length == 3)
            id = keys[2];
          UIPortlet uiPortlet = new UIPortlet(portletApp, portletName, id);
          uiPortlet.getPortletModel().setPortletStyle(style);
          uiPortlet.getPortletModel().setDecorator(getDecorator());
          children.add(uiPortlet);
          uiPage.buildTreeModel(null);
          configService.savePage(uiPage.getPageModel());
          uiPage.clearComponentModified();
          uiPage.setComponentDirty(false);
        }
      }
    } catch (Throwable t) {
      t.printStackTrace();
    }
  }

  private void registerListeners() {
    addActionListener(NodeActionListener.class, PortalConstants.SAVE_NODE_ACTION);
    addActionListener(NodeActionListener.class, PortalConstants.BROWSE_PAGE_ACTION);
    addActionListener(NodeActionListener.class, PortalConstants.SHOW_NAVIGATION_ACTION);
    addActionListener(PageActionListener.class, PortalConstants.SAVE_PAGE_ACTION);
    addActionListener(PortalActionListener.class, PortalConstants.EDIT_PORTAL_ACTION);
    addActionListener(PortalActionListener.class, PortalConstants.SAVE_PORTAL_ACTION);
    addActionListener(PortalActionListener.class, PortalConstants.CHANGE_LANGUAGE_ACTION);
    addActionListener(PortalActionListener.class, PortalConstants.DRAG_DROP_ACTION);
  }

  public String getIdPrefix() {  return "portal"; }

  public Component getComponentModel() { return componentModel_; }

  public PortalConfig getPortalConfigModel() {   return componentModel_; }

  public PortalConfig getEditablePortalConfigModel() {
    if (!modified_)
      componentModel_ = (PortalConfig) componentModel_.softCloneObject();
    return componentModel_;
  }

  public String getOwner() {  return portalOwner_; }

  public String getUserCss() {   return userCss_;  }
  public String getJavaRendererType() {  return "PortalRenderer"; }

  /** Override default set editable in UIBasicComponent */
  // public void setComponentAdminRole(boolean b) { componentAdminRole_ = b ; }
  public boolean isDirty() {
    return dirty_;
  }

  /** Override default setDirty in UIBasicComponent */
  public void setComponentDirty(boolean b) {  dirty_ = b; }

  public boolean hasEditPortalCapability() {  return editPortalCapability_;  }

  public void setEditPortalCapability(boolean b) {   editPortalCapability_ = b; }

  public boolean hasEditNavigationCapability() {  return editNavigationCapability_; }

  public void setEditNavigationCapability(boolean b) {  editNavigationCapability_ = b; }

  public int getPortalMode() {   return portalMode_;  }

  public void setPortalMode(int mode) throws Exception {
    portalMode_ = mode;
    if (mode == PORTAL_EDIT_MODE)
      PortalMode.changePortalEditMode(this);
    else if (mode == PORTAL_EDIT_PAGE_MODE)
      PortalMode.changePortalEditPageMode(this);
    else if (mode == PORTAL_EDIT_NAVIGATION_MODE)
      PortalMode.changePortalEditNavigationMode(this);
    else
      PortalMode.changePortalViewlMode(this);
  }

  public Node getRootNode() {    return root_;  }

  public Node getSelectedNode() {   return selectNode_; }

  public void setSelectedNode(Node node) {   selectNode_ = node; }

  public void setSelectedPage(Node node) {
    if (selectNode_ != null) {
      selectNode_.setSelectedPath(false);
      selectNode_ = node;
      selectNode_.setSelectedPath(true);
      PortalComponentCache cache = (PortalComponentCache) SessionContainer
          .getComponent(PortalComponentCache.class);
      UIPage uiPage = cache.getUIPage(selectNode_, this);
      PortalLocale.changeLocale(uiPage);
      setBodyComponent(uiPage);
    }
  }

  public void removeSelectedNode() {
    Node parent = selectNode_.getParent();
    if (parent != null && !selectNode_.isShare()) {
      parent.removeChild(selectNode_.getUri());
      setSelectedPage(parent);
    }
  }

  public UIBody findUIBody() {
    return (UIBody) findDescendantOfType(UIBody.class);
  }

  public UIPage getCurrentUIPage() {
    UIPage uipage = (UIPage) findDescendantOfType(UIPage.class);
    if (uipage != null)  return uipage;
    PortalComponentCache cache = 
      (PortalComponentCache) SessionContainer.getComponent(PortalComponentCache.class);
    return cache.getUIPage(selectNode_, this);
  }

  public void setBodyComponent(UIComponent component) {
    findUIBody().setBodyComponent(component);
    if (component instanceof UIPage) {
      UIPage uiPage = (UIPage) component;
      if (portalMode_ == ExoPortal.PORTAL_EDIT_PAGE_MODE) {
        PortalMode.changePageMode(uiPage, UIPortalComponent.COMPONENT_EDIT_MODE);
      } else {
        PortalMode.changePageMode(uiPage, UIPortalComponent.COMPONENT_VIEW_MODE);
      }
    }
    userCss_ = new PortalSkin().getCSS(this);
  }

  public void setLastBodyComponent() {
    findUIBody().setLastBodyComponent();
    userCss_ = new PortalSkin().getCSS(this);
  }

  final public void processDecodes(FacesContext context) {
    RequestInfo pparams = (RequestInfo) SessionContainer.getComponent(RequestInfo.class);
    String targetComponent = pparams.getTargetComponentId();
    if (targetComponent != null) {
      UIComponent uiTargetComponent = (UIComponent) findComponentById(targetComponent);
      if (uiTargetComponent == null) {
        uiTargetComponent = this ;
      } 
      if (uiTargetComponent == this)
        decode(context);
      else
        uiTargetComponent.processDecodes(context);
    } else {
      findUIBody().processDecodes(context);
    }
  }

  protected String getComponentIdPrefix() {  return "";  }

  public Style getDecoratorStyle(SkinConfigService service, String renderer, String style) {
    return service.getPortalDecoratorStyle(renderer, style);
  }

  public void changeLocale(ResourceBundle res) {
    root_.visit(new ResolveLabelVisitor(res));
  }

  public void buildTreeModel(Container parent) { }


  public void clearMessages() {   information_.clearMessages();  }
  public List getMessages() {   return information_.getMessages(); }
  public boolean hasMessage() {   return information_.hasMessage(); }
  public Information getInformation()  { return information_ ; }

  public boolean hasQueuEvent() {   return queueEvent_; }

  public void setQueueEvent(boolean b) {   queueEvent_ = b;  }

  public void queueEvent(javax.faces.event.FacesEvent event) {
    queueEvent_ = true;
    super.queueEvent(event);
  }

  class ResolveLabelVisitor implements NodeVisitor {
    ResourceBundle res_;

    public ResolveLabelVisitor(ResourceBundle res) {
      res_ = res;
    }

    public void visit(Node node) {   node.setResolvedLabel(res_);   }
  }

  public String getTemplatePath() {
    return "org/exoplatform/portal/faces/renderer/html/portal/UIPortal.vm" ;
  }
}