/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.faces.component;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;
import javax.faces.context.FacesContext;
import javax.faces.event.AbortProcessingException;
import javax.faces.event.ActionListener;
import javax.faces.event.FacesEvent;
import org.exoplatform.Constants;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.portal.session.RequestInfo;
import org.exoplatform.services.portal.model.Component;
import org.exoplatform.services.portal.model.Container;
import org.exoplatform.services.portal.skin.SkinConfigService;
import org.exoplatform.services.portal.skin.model.Style;
/**
 * Date: Aug 11, 2003
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIBasicComponent.java,v 1.10 2004/09/26 02:25:46 tuan08 Exp $
 */
abstract public class UIPortalComponent extends UIExoCommand {
  final static public short  FLOAT_RIGHT = 0 ;
  final static public short  FLOAT_BOTTOM = 1 ;
  
  final static public  int COMPONENT_VIEW_MODE = 1 ;
  final static public int COMPONENT_EDIT_MODE = 2 ;

  protected String decorator_ ;
  private short float_ ; 
  protected boolean componentAdminRole_ = false ;
  protected boolean modified_ ;
  protected int mode_   = COMPONENT_VIEW_MODE  ;
  transient private UIComponentCommandChain commandChain_ ;

  public short getFloat() { return float_ ; }
  public void  setFloat(short val) { float_ = val ;}
  
  abstract public Component getComponentModel() ;
  
  protected void setBasicProperties(Component componentModel, String defaultStyle) {
    setDecorator(componentModel.getDecorator(), defaultStyle) ;
    setRendererType(getJavaRendererType()) ;
    setId(componentModel.getId()) ;
  }
  
  public void updateChange() {
  	Component model = getComponentModel() ;
  	setDecorator(model.getDecorator(), "default") ;
  }
  
  public String getDecorator() { return decorator_; }
  public void   setDecorator(String decorator, String defaultDecorator) { 
    if(decorator == null || decorator.length() == 0) {
      decorator_ = defaultDecorator ;
    } else {
      decorator_ = decorator ; 
    }
  }
  
  public int getMode() { return mode_ ; }
  public void setMode(int mode) {  mode_ = mode ; }
  
  public boolean hasComponentAdminRole() { return componentAdminRole_ ; }
  public void    setComponentAdminRole(boolean b) { componentAdminRole_ = b ; }
  
  public void   setRendererType(String type) { 
    String rendererType = type ;
    if(type == null || type.length() == 0 || "default".equals(type)) {
      rendererType = getDefaultRendererName() ;
    }
    super.setRendererType(rendererType) ;
  }

  public boolean isComponentModified() { return modified_ ; }
  public void setComponentModified(boolean b) {  
  	modified_ = b ;
    setComponentDirty(b) ;
  }
  
  public void clearComponentModified() {
  	modified_ = false ;
  	List children = getChildren() ;
  	for(int i = 0; i < children.size(); i++) {
  		Object uichild =  children.get(i) ;
      if(uichild instanceof UIPortalComponent)
  		((UIPortalComponent)uichild).clearComponentModified() ;
  	}
  }
  
  public void setComponentDirty(boolean b) {  
    UIPortalComponent uiParent = (UIPortalComponent) getParent() ;
    uiParent.setComponentDirty(b) ;
  }

  final public void decode(FacesContext context) {
    getRenderer(context).decode(context, this) ;
  }
  
  final public void setId(String id) {
    if (id == null || id.length() == 0) {
    	id = new String(getIdPrefix()  + Integer.toString(hashCode())); 
    }
    super.setId(id) ;
  }
  
  public void removeChild(UIPortalComponent child) {
    List children = getChildren() ;
    children.remove(child) ;
    setComponentModified(true) ;
  }
  
  public void insertChildBefore(UIPortalComponent newChild, UIPortalComponent child) {
    List children = getChildren() ;
    if(child == null ||  children.size() == 0) {
      children.add(newChild) ;
      return ;
    }
    for(int i = 0; i < children.size(); i++) {
      Object uichild = children.get(i) ;
      if(uichild ==  child) {
        //the list implement by sun has a bug , the add(int pos, component)
        //do not  insert the component into  the  right position
        List list = new ArrayList(children) ;
        list.add(i, newChild) ;
        children.clear() ;
        children.addAll(list) ;
        return ;
      }
    }
    throw new RuntimeException("cannot  find the child: " +  child.getId()) ;
  }
  
  public String getBaseURL() {
    if (baseURL_ == null) {
      RequestInfo rinfo = (RequestInfo)SessionContainer.getComponent(RequestInfo.class);
      baseURL_ = rinfo.getOwnerURI();
      baseURL_ += "?" + Constants.COMPONENT_PARAMETER + "=" + getId() ;
    }
    return baseURL_ ;
  }
  
  public void changeLocale(ResourceBundle res) {  }
  
  public String getDisplayTitle() { return getId() ; }
  
  public String getDefaultRendererName() { return  "default" ; }
  abstract public String getJavaRendererType() ;
  abstract public String getIdPrefix() ;
  abstract public Style getDecoratorStyle(SkinConfigService service, String renderer, String style) ;
  abstract public void buildTreeModel(Container parent) ;
  
  public String getFamily() { 
    return "org.exoplatform.portal.faces.component.UIPortalComponent" ; 
  }
  
  final public void encodeBegin(FacesContext context) throws IOException { }
  
  final public void encodeEnd(FacesContext context) throws IOException { }
  
  final public void encodeChildren(FacesContext context) throws IOException {   
    if(commandChain_ == null) {
      commandChain_ = 
        (UIComponentCommandChain) PortalContainer.getComponent(UIComponentCommandChain.class) ;
    }
    try {
      commandChain_.renderComponent(this, context) ;
    } catch (Throwable t) {
      t.printStackTrace() ;
      throw (IOException) t ;
    }
  }
  
  public void broadcast(FacesEvent event) throws AbortProcessingException {
    ActionListener[] listener = this.getActionListeners() ;
    for(int i = 0; i < listener.length; i++) {
      if (event.isAppropriateListener(listener[i])) {
        event.processListener(listener[i]);
      }
    }
  }
}
