/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portal.faces.component;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.faces.component.UIComponent;
import javax.faces.event.PhaseId;

import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.user.component.UIExoPermissionBrowser;
import org.exoplatform.faces.user.component.UIExoPermissionSelector;
import org.exoplatform.services.organization.OrganizationService;
import org.exoplatform.services.portal.model.Portlet;
import org.exoplatform.services.portal.skin.SkinConfigService;
import org.exoplatform.services.portal.skin.model.Decorator;
import org.exoplatform.services.portal.skin.model.Style;
import org.exoplatform.services.portletcontainer.pci.ExoWindowID;
/**
 * Sat, Jan 03, 2004 @ 11:16 
 * @author: Tuan Nguyen
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIPortletForm.java,v 1.10 2004/09/30 01:00:05 tuan08 Exp $
 */
public class UIPortletForm extends UISimpleForm implements SelectorHandler {
static String  SEARCH_GROUP = 
    org.exoplatform.faces.core.Util.encodeActionPhase("searchGroup", PhaseId.APPLY_REQUEST_VALUES) ;
  final static String UPDATE_RENDERER_ACTION = "updateRenderer";
  final static String ID = "id" , TITLE = "title" , RENDERER = "renderer",
               STYLE = "style" , DECORATOR = "decorator", HEIGHT = "height", 
               WIDTH = "width" , EDIT_PREF_PERMISSION = "editPrefPermission",
               SHOW_INFO_BAR = "showInfoBar", SHOW_WINDOW_STATE = "showWindowState",
               SHOW_PORTLET_MODE = "showPortletMode";
  
  private UIPortlet uiEditingPortlet_ ; 
  private UIComponent returnComponent_ ;
  private SkinConfigService skinService_ ;
  
  private List decoratorOptions_ ;
  private List portletStyleOptions_ ;
  private List rendererOptions_ ;
  
  public UIPortletForm(SkinConfigService configService, 
                       OrganizationService orgService) throws Exception {
    super("portletForm", "post", null) ;
    setId("UIPortletForm");
    setRendererType("SimpleFormVelocityRenderer") ;
    setClazz("UIExoForm") ;
    skinService_ = configService ;
    add(new UIStringInput(ID, "").setEditable(false)) ;
    add(new UIStringInput(TITLE, "")) ;
    add(new UISelectBox(STYLE, "", null)) ;
    add(new UISelectBox(DECORATOR, "", null)) ;
    UISelectBox uiSelect = new UISelectBox(RENDERER,"" , null) ;
    uiSelect.setUpdateOnChangeAction(UPDATE_RENDERER_ACTION) ;
    add(uiSelect) ;
    add(new UIStringInput(HEIGHT, "")) ;
    add(new UIStringInput(WIDTH, "")) ;
    add(new UIExoPermissionSelector(EDIT_PREF_PERMISSION, null)) ;
    add(new UICheckBox(SHOW_INFO_BAR, "true")) ;
    add(new UICheckBox(SHOW_WINDOW_STATE, "true")) ;
    add(new UICheckBox(SHOW_PORTLET_MODE, "true")) ;
    rendererOptions_ = new ArrayList(5) ;
    portletStyleOptions_ = new ArrayList(5) ;
    decoratorOptions_ = new ArrayList(5) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION) ;
    addActionListener(SaveActionListener.class, SAVE_ACTION) ;
    addActionListener(UpdateRendererActionListener.class, UPDATE_RENDERER_ACTION) ;
    addActionListener(SearchGroupActionListener.class, SEARCH_GROUP) ;
  }
  
  public String getSearchGroupAction() { return SEARCH_GROUP; }
  
  public void setEditingPortlet(UIComponent returnComponent, UIPortlet uiPortlet) {
    returnComponent_ = returnComponent ;
    uiEditingPortlet_ = uiPortlet ; 
    rendererOptions_.clear() ;
    portletStyleOptions_.clear() ;
    decoratorOptions_.clear() ;
    ExoWindowID windowID = uiEditingPortlet_.getWindowId() ;
    String portletId = windowID.getPortletApplicationName() + "/" + windowID.getPortletName()  ;
    Portlet model = uiPortlet.getPortletModel() ;
    Iterator decoratorIterator = skinService_.getPortletDecorators().iterator() ;
    String currentRenderer = uiPortlet.getRendererType() ;
    Decorator selectDecorator = null ;
    while(decoratorIterator.hasNext()) {
      Decorator decorator = (Decorator) decoratorIterator.next() ;
      String rendererType = decorator.getRendererType() ;
      rendererOptions_.add(new SelectItem(rendererType, rendererType)) ;
      if (selectDecorator == null)  {
        selectDecorator = decorator ;
      }else if(currentRenderer.equals(rendererType)) {
        selectDecorator = decorator ;
      }
    }

    List styles = selectDecorator.getStyles() ;
    for (int i = 0; i < styles.size(); i++) {
      Style style = (Style) styles.get(i) ;
      String name = style.getName() ;
      decoratorOptions_.add(new SelectItem(name, name)) ;
    }
    
    List portletStyles = skinService_.getPortletStyles(portletId) ;
    if(portletStyles == null)  {
      portletStyles = skinService_.getPortletStyles("default") ;
    }
    for (int i = 0; i < portletStyles.size(); i++) {
      Style style = (Style) portletStyles.get(i) ;
      String name = style.getName() ;
      portletStyleOptions_.add(new SelectItem(name, name)) ;
    }

    getUIStringInput(ID).setValue(uiPortlet.getId()) ;
    getUIStringInput(TITLE).setValue(model.getTitle()) ;
    
    UISelectBox uiSelect = getUISelectBox(RENDERER) ;
    uiSelect.setOptions(rendererOptions_);
    uiSelect.setValue(currentRenderer);
    
    uiSelect = getUISelectBox(DECORATOR) ;
    uiSelect.setOptions(decoratorOptions_);
    uiSelect.setValue(model.getDecorator());
    
    uiSelect = getUISelectBox(STYLE) ;
    uiSelect.setOptions(portletStyleOptions_);
    uiSelect.setValue(model.getPortletStyle());
    
    getUIStringInput(WIDTH).setValue(model.getWidth()) ;
    getUIStringInput(HEIGHT).setValue(model.getHeight()) ;
    getUICheckBox(SHOW_INFO_BAR).setSelect(model.getShowInfoBar()) ;
    getUICheckBox(SHOW_WINDOW_STATE).setSelect(model.getShowWindowState()) ;
    getUICheckBox(SHOW_PORTLET_MODE).setSelect(model.getShowPortletMode()) ;
    
    UIExoPermissionSelector editPrefPerm = (UIExoPermissionSelector) getUIInput(EDIT_PREF_PERMISSION) ;
    editPrefPerm.setPermission(model.getEditPreferencesPermission()) ;
  }
  
 public void cancel(Selector selector) {
    UIPortal uiPortal = (UIPortal) selector.getAncestorOfType(UIPortal.class);
    uiPortal.setBodyComponent(this);
  }

  public void select(Selector selector) throws Exception {
    UIExoPermissionSelector editPrefPerm = 
      (UIExoPermissionSelector) getUIInput(EDIT_PREF_PERMISSION) ;
    editPrefPerm.setPermission(selector.getInputValue()) ;
    UIPortal uiPortal = (UIPortal) selector.getAncestorOfType(UIPortal.class);
    uiPortal.setBodyComponent(this);
  }
  
  static public class UpdateRendererActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletForm uiForm = (UIPortletForm) event.getSource() ;
      String selectRenderer = uiForm.getUISelectBox(RENDERER).getValue() ;
      Decorator selectDecorator =  uiForm.skinService_.getPortletDecorator(selectRenderer) ; 
      uiForm.decoratorOptions_.clear() ;
      List styles = selectDecorator.getStyles() ;
      String decoratorDefaultStyle = "default" ;
      for (int i = 0; i < styles.size(); i++) {
        Style style = (Style) styles.get(i) ;
        String name = style.getName() ;
        uiForm.decoratorOptions_.add(new SelectItem(name, name)) ;
        if(i == 0) decoratorDefaultStyle = name ;
      }
      uiForm.getUISelectBox(RENDERER).setValue(selectRenderer);
      uiForm.getUISelectBox(DECORATOR).setOptions(uiForm.decoratorOptions_);
      uiForm.getUISelectBox(DECORATOR).setValue(decoratorDefaultStyle);
    }
  }
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletForm uiForm = (UIPortletForm) event.getSource() ;
      UIPortal uiPortal = (UIPortal) uiForm.getAncestorOfType(UIPortal.class);
      uiPortal.setBodyComponent(uiForm.returnComponent_) ;
    }
  }
  
  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletForm uiForm = (UIPortletForm) event.getSource() ;
      UIPortal uiPortal = (UIPortal) uiForm.getAncestorOfType(UIPortal.class);
      Portlet model = uiForm.uiEditingPortlet_.getEditablePortletModel() ;
      String portletTitle = uiForm.getUIStringInput(TITLE).getValue() ;
      if(portletTitle == null || portletTitle.length() == 0) portletTitle  = null ;
      model.setTitle(portletTitle) ;
      model.setDecorator(uiForm.getUISelectBox(DECORATOR).getValue()) ;
      model.setPortletStyle(uiForm.getUISelectBox(STYLE).getValue()) ;
      model.setRenderer(uiForm.getUISelectBox(RENDERER).getValue()) ;
      model.setWidth(uiForm.getUIStringInput(WIDTH).getValue()) ;
      model.setHeight(uiForm.getUIStringInput(HEIGHT).getValue()) ;
      model.setShowInfoBar(uiForm.getUICheckBox(SHOW_INFO_BAR).getSelect()) ;
      model.setShowWindowState(uiForm.getUICheckBox(SHOW_WINDOW_STATE).getSelect()) ;
      model.setShowPortletMode(uiForm.getUICheckBox(SHOW_PORTLET_MODE).getSelect()) ;
      model.setId(uiForm.uiEditingPortlet_.getId()) ;
      UIExoPermissionSelector editPrefPerm = 
        (UIExoPermissionSelector) uiForm.getUIInput(EDIT_PREF_PERMISSION) ;
      model.setEditPreferencesPermission(editPrefPerm.getPermission()) ;
      String id = uiForm.getUIStringInput(ID).getValue()  ;
      ExoWindowID windowId = uiForm.uiEditingPortlet_.getWindowId() ;
      if(id == null || id.length() == 0) id =  windowId.getPortletName() ;
      windowId.setUniqueID(id) ;
      model.setWindowId(windowId.generatePersistenceId()) ;
      uiForm.uiEditingPortlet_.setId(id);  
      uiForm.uiEditingPortlet_.setDisplayTitle(portletTitle);  
      uiForm.uiEditingPortlet_.updateChange() ;
      uiForm.uiEditingPortlet_.setComponentModified(true) ;
      uiPortal.setBodyComponent(uiForm.returnComponent_) ;
    }
  }
  
 static public class SearchGroupActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPortletForm uiForm = (UIPortletForm) event.getComponent();
      UIPortal uiPortal = (UIPortal) uiForm.getAncestorOfType(UIPortal.class);
      UIExoPermissionBrowser uiGroupBrowser = 
        (UIExoPermissionBrowser)PortalComponentCache.findPortalComponent(UIExoPermissionBrowser.class);
      uiGroupBrowser.setHandler(uiForm) ;
      uiPortal.setBodyComponent(uiGroupBrowser) ;
    }
  }
}