/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.indexing.component;

import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
  import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.indexing.FileIndexerPlugin;
import org.exoplatform.services.indexing.IndexingService;

/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Aug 27, 2004
 * @version $Id: UIFileIndexerPlugin.java,v 1.2 2004/09/17 16:59:12 tuan08 Exp $
 */
public class UIFileIndexerPlugin extends  UISimpleForm {
  final static public String INDEX_DIRECTORY = "index" ;
  final static public String REINDEX_DIRECTORY = "reindex" ;
  
  private UIStringInput directoryInput_ ;
  private UIStringInput acceptExtInput_ ;
  private UICheckBox recursiveInput_ ;
  private FileIndexerPlugin indexer_ ;
  
  public UIFileIndexerPlugin(IndexingService iservice) throws Exception {
    super("fileIndexerPlugin", "post", null) ;
  	setId(FileIndexerPlugin.IDENTIFIER) ;
  	setRendererType("SimpleFormVelocityRenderer") ;
    indexer_ = (FileIndexerPlugin)iservice.getIndexerPlugin(FileIndexerPlugin.IDENTIFIER) ;    
    directoryInput_ = new UIStringInput("directory", "") ;
    acceptExtInput_ = new UIStringInput("acceptExt", "html, txt, xml, java") ;
    recursiveInput_ = new UICheckBox("recursive", "true") ;
    recursiveInput_.setSelect(true) ;
    add(directoryInput_) ;
    add(acceptExtInput_) ;
    add(recursiveInput_) ;
    addActionListener(ShowListIndexerPluginActionListener.class, CANCEL_ACTION) ;
    addActionListener(IndexActionListener.class, INDEX_DIRECTORY) ;
    addActionListener(ReindexActionListener.class, REINDEX_DIRECTORY) ;
  }
  
  public String getIndexAction() { return INDEX_DIRECTORY ; }
  public String getReIndexAction() { return REINDEX_DIRECTORY ; }
  
  static public class IndexActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIFileIndexerPlugin uiComponent = (UIFileIndexerPlugin) event.getComponent() ;
      String directory = uiComponent.directoryInput_.getValue() ;
      String acceptExt =  uiComponent.acceptExtInput_.getValue() ;
      String[] ext = acceptExt.split(",") ;
      boolean recursive = uiComponent.recursiveInput_.getSelect();
      System.out.println("\n\nindex: " + directory + " , recursive : " + recursive + "\n\n") ;
      uiComponent.indexer_.indexDirectory(directory, null ,ext, recursive) ;
    }
  }
  
  static public class ReindexActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UIFileIndexerPlugin uiComponent = (UIFileIndexerPlugin) event.getComponent() ;
      String directory = uiComponent.directoryInput_.getValue() ;
      String acceptExt =  uiComponent.acceptExtInput_.getValue() ;
      String[] ext = acceptExt.split(",") ;
      boolean recursive = "true".equals( uiComponent.recursiveInput_.getValue());
      System.out.println("\n\nindex: " + directory + " , recursive : " + recursive + "\n\n") ;
      uiComponent.indexer_.reindexDirectory(directory, null, ext, recursive) ;
    }
  }
}