/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.display.statik.component;

import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;

import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.session.ExoPortal;
import org.exoplatform.portlets.content.ContentUtil;
import org.exoplatform.portlets.content.display.model.ContentConfig;
import org.exoplatform.services.portal.model.Node;
import org.exoplatform.services.portletcontainer.ExoPortletRequest;
/**
 * Wed, Dec 22, 2003 @ 23:14
 * @author: Tuan Nguyen
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UIContentConfig.java,v 1.4 2004/11/03 04:24:55 tuan08 Exp $
 */
public class UIContentConfig extends UIExoCommand  {
	
	final static public String[] DEFAULT_VALUES = { "title=", "uri=", "encoding="};
	
	private static final String NEW_ACTION = "new";
	
	private static Parameter[] NEW_PARAMS = { new Parameter(ACTION , NEW_ACTION) };
	private static Parameter[] EDIT_PARAMS = { new Parameter(ACTION , EDIT_ACTION) };
	private static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_ACTION) };
	
	Map configs_;
  boolean modificationAllowed_;
	
	public UIContentConfig() {
	  setRendererType("VelocityRenderer") ;		
		setId("UIContentConfig");
		updateConfigs();
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ;
    addActionListener(EditActionListener.class, EDIT_ACTION) ;
    addActionListener(NewActionListener.class, NEW_ACTION) ;
	}
	
	void updateConfigs() {
		ExternalContext eContext = FacesContext.getCurrentInstance().getExternalContext();
		PortletRequest request = (PortletRequest) eContext.getRequest();    
		PortletPreferences prefs = request.getPreferences();
		Enumeration e = prefs.getNames();
		configs_ = new HashMap();
		while (e.hasMoreElements()) {
			String name = (String) e.nextElement();
			String[] values = prefs.getValues(name, DEFAULT_VALUES);
			ContentConfig contentConfig = new ContentConfig(name, values);
			configs_.put(name, contentConfig);
		}
	}
	
	public Parameter[] getNewParams() { return NEW_PARAMS; }
	public Parameter[] getEditParams() { return EDIT_PARAMS; }
	public Parameter[] getDeleteParams() { return DELETE_PARAMS; }
	
	
	public Collection getConfigs() {
		return configs_.values();
	}
	  
  public boolean isModificationAllowed() {
    return modificationAllowed_;
  }  
  
  public void setModificationAllowed(boolean modificationAllowed) {
    modificationAllowed_ = modificationAllowed;
  }
	
	public void saveContentConfig(String action, ContentConfig config) throws Exception {
		FacesContext context = getFacesContext();
		ExoPortletRequest request = (ExoPortletRequest) context.getExternalContext().getRequest();
		PortletPreferences prefs = request.getPreferences();
		
		String uri = config.getUri();
	  if (uri == null) {
		  uri = "";
		  config.setUri(uri);
		}
		
		String name = config.getName();
		String[] tmp = new String[3];
		tmp[0] = "title=" + config.getTitle();
		tmp[1] = "uri=" + uri;
		tmp[2] = "encoding=" + config.getEncoding();
		
		prefs.setValues(name, tmp);
		prefs.store();
		updateConfigs();
	}
	
	public String getUri(ExoPortletRequest request) {
    ExoPortal portal = (ExoPortal)SessionContainer.getComponent(ExoPortal.class) ;
    Node selectedNode = portal.getSelectedNode();        
    String uri = selectedNode.getUri();
    if("/".equals(uri)){
      uri = "/" + selectedNode.getName();          
    }
    String uniqueID = request.getWindowID().getUniqueID();
    return uri += "/" + uniqueID;
	  
	}
  
	static public class DeleteActionListener extends ExoActionListener {
		public void execute(ExoActionEvent event) throws Exception {
		  UIContentConfig uiConfig = (UIContentConfig) event.getComponent() ;
      String name =  event.getParameter(OBJECTID) ;
			ExoPortletRequest request = 
        (ExoPortletRequest) FacesContext.getCurrentInstance().getExternalContext().getRequest();
      if(uiConfig.isModificationAllowed()) {
        String uri = uiConfig.getUri(request) + "/" + name;
        //ContentUtil.removeContent(uri);
      }      
			PortletPreferences prefs = request.getPreferences();
			prefs.reset(name);
	    prefs.store();
      uiConfig.updateConfigs();
		}
	}
  
  static public class EditActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentConfig uiConfig = (UIContentConfig) event.getComponent() ;
      String name = event.getParameter(OBJECTID);      
      ContentConfig config = (ContentConfig) uiConfig.configs_.get(name);
      UIContentConfigForm uiConfigForm = 
        (UIContentConfigForm)uiConfig.getSibling(UIContentConfigForm.class);
      uiConfigForm.setContentConfig(config);
      uiConfig.setRenderedSibling(UIContentConfigForm.class) ;
    }
  }
  
  static public class NewActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIContentConfig uiConfig = (UIContentConfig) event.getComponent() ;
      UIContentConfigForm uiConfigForm = 
        (UIContentConfigForm)uiConfig.getSibling(UIContentConfigForm.class);
      uiConfigForm.setContentConfig(null);
      uiConfig.setRenderedSibling(UIContentConfigForm.class) ;
    }
  }

}
