/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.impl.converter.v11tov2;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.exoplatform.services.portal.model.Permission;
import org.exoplatform.services.portal.model.PortletPreferences;
import org.exoplatform.services.portal.model.Preference;
import org.exoplatform.xml.object.XMLCollection;
import org.exoplatform.xml.object.XMLEntry;
import org.exoplatform.xml.object.XMLField;
import org.exoplatform.xml.object.XMLMap;
import org.exoplatform.xml.object.XMLObject;
import org.exoplatform.xml.object.XMLValue;

/**
 * Created by The eXo Platform SARL
 * Author : Tuan Nguyen
 *          tuan08@users.sourceforge.net
 * Nov 24, 2005
 */
public class BaseConverter {
  
  protected Permission convertObjectPermission(XMLObject xmlobject, String fieldName) throws Exception {
    Object obj = xmlobject.getFieldValue( fieldName);
    if( obj == null) return null;
    String valuePermistion = obj.toString();   
    String valueGroup = "/guest";
    if(  valuePermistion.trim().equals("owner")) valueGroup = "/user";
    Permission permision = new Permission( valueGroup, valuePermistion);    
    return permision;
  }
  
  protected void setField(
      XMLObject xml, String name, Object pattern, Object value) throws Exception {   
    boolean set = false;  
    if( name.equals( "portletPreferences")){
      String [] p = (String[]) pattern;
      String [] v = (String[]) value;
      PortletPreferences pre = convertPreferences( xml, p, v);  
      xml.removeField( name);
      xml.addField( name, pre.getClass(), pre) ; 
      return;
    }   
    if( name.equals( "windowId")){      
      if( value == null) return;
      String windowId = (String)xml.getFieldValue("windowId");      
      value = windowId.replaceAll( (String)pattern, (String)value);     
      set= true;
    }else if( pattern == null) set = true;    
    else if( pattern.toString().equals("*")) set = true;   
    else if( pattern.toString().equals( String.valueOf(xml.getFieldValue(name)))) set = true;
    if( set == false) return;    
    xml.removeField( name);
    if( value == null) return;
    xml.addField( name, value.getClass(), value); 
  } 
  
  protected void convertPermission(XMLObject xmlobject, String type) throws Exception { 
    Permission per = convertObjectPermission( xmlobject, type);
    xmlobject.removeField( type);
    xmlobject.addField( type, Permission.class.getClass(), per);  
  }  
  
  protected PortletPreferences convertPreferences(  XMLObject xml, String[] p, String[] v)throws Exception{
    PortletPreferences  pre = new PortletPreferences (); 
    ArrayList<Preference> list = new ArrayList<Preference>();   
    XMLMap map = xml.getField("portletPreferences").getMap();
    if( map != null) {  
      Iterator i = map.getEntryIterator();    
      while( i.hasNext()){
        XMLEntry entry = (XMLEntry)i.next();          
        XMLObject obj = entry.getValue().getObject();  
        list.add( convertPreference(obj, p, v));
      }    
    }
    if( v != null && list.size() == 0) {
      list.add( convertPreference( null, null, v));
    }
    pre.setPreferences( list);
    return pre;
  }
  
  protected Preference convertPreference( XMLObject xml, String[] p, String[] v)throws Exception {   
    Preference  pre = new Preference();
    ArrayList<String> list = null;
    if( xml != null){
      list  = (ArrayList)xml.getFieldValue("values");     
      pre.setName( (String)xml.getFieldValue("name"));
      pre.setReadOnly( (Boolean)xml.getFieldValue("readOnly"));
      if( v == null){
        pre.setValues( list);
        return pre;
      }
    }else{
      pre.setReadOnly( true);
      pre.setName("template");
    }
    if( list  == null) list  = new ArrayList<String>();
    if( p != null){
      for( int i = 0; i< p.length; i++){        
        if( p[i] != null ){
          for( int j= 0; j<list.size(); j++)
            if( list.get( j).toString().indexOf( p[i])> -1 ) {
              if ( v[i] != null )list.set( j, v[i]);              
              else list.remove( j);  
            }
        }else if( v[i] != null)  list.add( v[i]);    
      }
    }else if( v !=  null)
      for( int i = 0; i< v.length; i++)
        if ( v[i] != null)  list.add( v[i]);    
      
    pre.setValues( list);    
    return pre;
  } 
  
}
