/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.services.portal.log.impl;

import java.io.Serializable;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import org.exoplatform.commons.utils.PageList;
import org.exoplatform.container.component.BaseComponentPlugin;
import org.exoplatform.container.xml.InitParams;
import org.exoplatform.container.xml.ValueParam;
import org.exoplatform.services.cache.CacheListener;
import org.exoplatform.services.cache.ExoCache;
import org.exoplatform.services.cache.FIFOExoCache;
import org.exoplatform.services.database.DBObjectPageList;
import org.exoplatform.services.database.HibernateService;
import org.exoplatform.services.database.ObjectQuery;
import org.exoplatform.services.log.LogService;
import org.exoplatform.services.portal.log.PortletLogData;
import org.exoplatform.services.portal.log.Query;
import org.hibernate.Session;

/**
 * @author Hung Nguyen (nqhungvn@users.sourceforge.net)
 * @since Sep 26, 2005
 * @version $Id$
 */
public class PortletActivitiesLogPlugin extends BaseComponentPlugin implements CacheListener{
  private FIFOExoCache[] portlets_ = new FIFOExoCache[24] ;
  private int currentTimeIndex_ = 0 ;
  private Date currentTimeSlot_ ;
  private HibernateService hservice_ ;
  private int cachesize_ ;
  private String uniqueId_ ;
  
  public PortletActivitiesLogPlugin(HibernateService hservice, InitParams params) {
    hservice_ = hservice ;
    ValueParam param = params.getValueParam("cachesize") ;
    cachesize_ = Integer.parseInt(param.getValue()) ; 
    updateTime() ;
  }
  
  public void updateTime() {
    GregorianCalendar  calendar = new  GregorianCalendar() ;
    calendar.set(GregorianCalendar.MINUTE, 0) ;
    calendar.set(GregorianCalendar.MILLISECOND, 0) ;
    calendar.set(GregorianCalendar.SECOND, 0) ;
    calendar.set(GregorianCalendar.MILLISECOND, 0) ;
    currentTimeSlot_ = calendar.getTime() ;
    currentTimeIndex_ = calendar.get(GregorianCalendar.HOUR_OF_DAY) ;
    uniqueId_ = "/" + String.valueOf(currentTimeSlot_.getYear() + 1900) + 
                "-" + String.valueOf(currentTimeSlot_.getMonth() + 1) +
                "-" + String.valueOf(currentTimeSlot_.getDate()) +
                "/" + String.valueOf(currentTimeIndex_);
    portlets_[currentTimeIndex_] = new FIFOExoCache(cachesize_) ;
    portlets_[currentTimeIndex_].addCacheListener(this) ;
  }
  public int getCurrentTimeIndex() { return currentTimeIndex_ ; }
  public void logPortletRender(String portletId, String portletAppId, long exeTime, boolean error, 
                               String eventType) {
    getOrCreatePortletLogData(portletId, portletAppId, exeTime, error, eventType);    
  }

  public void logPortletAction(String portletId, String portletAppId, long exeTime, boolean error, 
                               String eventType) {
    getOrCreatePortletLogData(portletId, portletAppId, exeTime, error, eventType);
  }
  
  synchronized private void getOrCreatePortletLogData(String portletId, String portletAppId, long exeTime, 
                                         boolean error, String eventType) {
    try {
      PortletLogDataImpl impl = 
    	  (PortletLogDataImpl) portlets_[currentTimeIndex_].get(portletAppId + "/" + portletId + uniqueId_) ;
      if(impl == null) {
        impl = new PortletLogDataImpl() ;
        if(eventType.equals("render")){
          impl.logPortletRender(portletAppId + "/" + portletId + uniqueId_, portletId, 
                              portletAppId + "/" + portletId , currentTimeSlot_, 
                              currentTimeIndex_, exeTime, error) ;
        }else {
          impl.logPortletAction(portletAppId + "/" + portletId + uniqueId_, portletId, 
                              portletAppId + "/" + portletId , currentTimeSlot_, 
                              currentTimeIndex_, exeTime, error) ;
        }
        
        portlets_[currentTimeIndex_].put(portletAppId + "/" + portletId + uniqueId_, impl) ;
      }else {
        if(eventType.equals("render")){
          impl.updatePortletRender(exeTime, error) ;
        }else { // eventType.equals("action")
          impl.updatePortletAction(exeTime, error) ;
        }        
      }      
    } catch(Exception ex) {
      ex.printStackTrace() ;      
    }
  }
  
  public void savePortletLogData(PortletLogData data) throws Exception {
	  Session session = hservice_.openSession() ;
	  PortletLogDataImpl impl = 
		  (PortletLogDataImpl) session.get(PortletLogDataImpl.class, data.getId()) ;
	  if(impl == null) {
		  impl = (PortletLogDataImpl) data ;
		  session.save(impl) ;
	  } else {
		  impl.merge((PortletLogDataImpl)data) ;
		  session.update(impl) ;
	  }
	  session.flush() ;
  }
  public PageList getPortletLogDatas(Query q) throws Exception {
    ObjectQuery oq = new ObjectQuery(PortletLogDataImpl.class);
    oq.addGT("timeSlot", q.getFromTimeSlot()) ;
    oq.addLT("timeSlot", q.getToTimeSlot()) ;
    oq.addGT("renderCounter", q.getCounter()) ;
    String componentId = q.getComponentId() ; 
    if(!componentId.equals("") || componentId.length() > 0) 
      oq.addLIKE("portletAppId", componentId) ;
    if(q.getOrder() != "" && q.getOrder().equals("Desc")){
      oq.setDescOrderBy("timeSlot") ;
    }else if(q.getOrder() != "" && q.getOrder().equals("Asc")){
      oq.setAscOrderBy("timeSlot") ;
    }
    
    return new DBObjectPageList(hservice_, oq);   
  }
  
  public List getPortletLogChartDatas(Query q) throws Exception {
    ObjectQuery oq = new ObjectQuery(PortletLogDataImpl.class);
    oq.addSelect("portletAppId") ;
    oq.addSUM("renderCounter") ;
    oq.addSUM("renderErrorCounter") ;
    oq.addSUM("sumRenderTime") ;
    oq.addSUM("processActionCounter") ;
    oq.addSUM("processActionErrorCounter") ;
    oq.addSUM("sumProcessActionTime") ;
    if(q.getMostAccess() != "" && q.getMostAccess().equals("most24")) {
      GregorianCalendar calendar = new GregorianCalendar() ;
      oq.addLT("timeSlot", calendar.getTime()) ;
      int dateBefore = calendar.get(GregorianCalendar.DATE) - 1;
      calendar.set(GregorianCalendar.DATE, dateBefore) ;
      oq.addGT("timeSlot", calendar.getTime()) ;
    }else if (q.getMostAccess() != "" && q.getMostAccess().equals("overall")){
      GregorianCalendar calendar = new GregorianCalendar() ;
      oq.addLT("timeSlot", calendar.getTime()) ;
    }else {
      oq.addGT("timeSlot", q.getFromTimeSlot()) ;
      oq.addLT("timeSlot", q.getToTimeSlot()) ; 
    }
    oq.addGT("renderCounter", q.getCounter()) ;
    String componentId = q.getComponentId() ; 
    if(!componentId.equals("") || componentId.length() > 0) 
      oq.addLIKE("portletAppId", componentId) ;
    oq.setGroupBy("portletAppId") ;    
    String groupQuery = oq.getHibernateGroupByQuery() ;
    if(q.getMostAccess() != "")
      groupQuery += " order by sum(o.processActionCounter) desc " ;
    Session session = hservice_.openSession() ;
    return session.createQuery(groupQuery).list() ;    
  }
  
  public List getCachedObjects(int timeIndex) throws Exception{
    List dataList = portlets_[timeIndex].getCachedObjects() ;
    portlets_[timeIndex].clearCache() ;
    return  dataList ;
  }
  
  public void  saveMonitorData() throws Exception {
    List cacheObjectList = getCachedObjects(currentTimeIndex_) ;
    for(int i = 0; i < cacheObjectList.size(); i++){
      PortletLogDataImpl data = (PortletLogDataImpl)cacheObjectList.get(i) ;
      savePortletLogData(data) ;
    }
  }
  
  public void savePrevTimeSlotMonitorData(int prevTimeSlot) throws Exception {
    List cacheObjectList = getCachedObjects(prevTimeSlot) ;
    for(int i = 0; i < cacheObjectList.size(); i++){
      PortletLogDataImpl data = (PortletLogDataImpl)cacheObjectList.get(i) ;
      savePortletLogData(data) ;
    }
  }
  
  public void onExpire(ExoCache arg0, Serializable arg1, Object arg2) throws Exception {
    PortletLogDataImpl data = (PortletLogDataImpl)arg2 ;
    savePortletLogData(data) ;
    if(arg0.get(arg1) != null)
      arg0.remove(arg1);
  }

  public void onRemove(ExoCache arg0, Serializable arg1, Object arg2) throws Exception {
  }

  public void onPut(ExoCache arg0, Serializable arg1, Object arg2) throws Exception {
  }

  public void onGet(ExoCache arg0, Serializable arg1, Object arg2) throws Exception {
  }

  public void onClearCache(ExoCache arg0) throws Exception {
  }
  
}
